// =============================================================================
// Framework User-Defined placeable
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Activate the placeable at specific times (dawn, day, dusk, night).

Variables
~~~~~~~~~

:ev:`DAWN`: *(int)* 1 if placeable is to be active at dawn.

:ev:`DAY`: *(int)* 1 if placeable is to be active during the day.

:ev:`DUSK`: *(int)* 1 if placeable is to be active at dusk.

:ev:`NIGHT`: *(int)* 1 if placeable is to be active during the night.

Examples
~~~~~~~~

Having a torch bracket light the torch during the night, and deactivating it
during the day.
*/


#include "fi_events"
#include "fi_object"
#include "fi_utils"


// -----------------------------------------------------------------------------
void OnHour()
{
  int nActivate = FALSE;
  nActivate |= (GetIsDawn()  && GetLocalInt(OBJECT_SELF, "DAWN"));
  nActivate |= (GetIsDay()   && GetLocalInt(OBJECT_SELF, "DAY"));
  nActivate |= (GetIsDusk()  && GetLocalInt(OBJECT_SELF, "DUSK"));
  nActivate |= (GetIsNight() && GetLocalInt(OBJECT_SELF, "NIGHT"));

  if (nActivate)
    SetObjectState(OBJECT_SELF, PLACEABLE_STATE_ACTIVATED);
  else
    SetObjectState(OBJECT_SELF, PLACEABLE_STATE_DEACTIVATED);
}


// -----------------------------------------------------------------------------
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_OBJECT_TRACKED:
    case EVENT_OBJECT_TRACKED_CALENDAR_HOUR:
      OnHour();
      return;
  }
}
