// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Close the door after timeout.

Variables
~~~~~~~~~

:ev:`AUTOCLOSE`: *(int)* Close the door after 12 seconds.

Scripts
~~~~~~~

:ev:`OnOpen`: :file:`fw_event`
*/


#include "fi_events"
#include "fi_console"

#include "nwnx_object"

const string AUTOCLOSE = "AUTOCLOSE";


// =============================================================================
void CloseDoor(object oDoor);
void CloseDoor(object oDoor)
{
  ActionCloseDoor(oDoor);
}

// =============================================================================
void main()
{
  int nEvent = GetEvent();

  if (nEvent != EVENT_OBJECT_OPENED)
    return;

  if (!NWNX_Object_GetDoorHasVisibleModel(OBJECT_SELF))
    return;

  float fValue = 12.0f;

  object oDoor = OBJECT_SELF;
  DelayCommand(fValue, CloseDoor(oDoor));
}
