// =============================================================================
// Framework User-Defined placeable
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Prevent resting in trigger area with optional feedback.

Variables
~~~~~~~~~

:ev:`NW_L_FEEDBACK`: *(int)* Speak this one liner from :file:`dlg_trg_norest` dialog when trying to rest.

Scripts
~~~~~~~

:ev:`OnEnter`: :file:`fw_event`

:ev:`OnExit`: :file:`fw_event`

:ev:`OnUserDefined`: :file:`fw_trg_norest`
*/

#include "fi_core"
#include "fi_events"

#include "nwnx_util"
#include "nwnx_creature"

const string NOREST = "NOREST";


// -----------------------------------------------------------------------------
void OnClientEnter();
void OnClientEnter()
{
  object oWho = GetEnteringObject();

  if (!GetIsPC(oWho))
    return;

  LogDebug(GetObjectInfo(oWho) + " entered a NOREST zone.");

  int nValue = GetLocalInt(OBJECT_SELF, NW_L_FEEDBACK);
  if (nValue)
    SetLocalInt(oWho, NOREST, nValue);
  else
    DeleteLocalInt(oWho, NOREST);
}


// -----------------------------------------------------------------------------
void OnClientExit();
void OnClientExit()
{
  object oWho = GetExitingObject();
  if (!GetIsPC(oWho))
    return;

  LogDebug(GetObjectInfo(oWho) + " left a NOREST zone.");

  DeleteLocalInt(oWho, NOREST);
}


// -----------------------------------------------------------------------------
void CancelRest(object oPC, int nFeedback = 0);
void CancelRest(object oPC, int nFeedback = 0)
{
  AssignCommand(oPC, ClearAllActions());
  if (nFeedback)
    SpeakOneLinerConversation("dlg_trg_norest", oPC);
}


// -----------------------------------------------------------------------------
void OnClientRest();
void OnClientRest()
{
  object oPC = GetLastPCRested();
  LogDebug(GetObjectInfo(oPC) + " ?");
  if (!GetIsPC(oPC))
    return;

  int nValue = GetLocalInt(oPC, NOREST);
  if (nValue == 0)
    return;
  CancelRest(oPC, nValue);
}


// =============================================================================
void main()
{
  object oPC;
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_PLAYER_REST_STARTED);
      return;

    case EVENT_TRIGGER_ENTER: OnClientEnter(); return;
    case EVENT_TRIGGER_EXIT:  OnClientExit(); return;
    case EVENT_PLAYER_REST_STARTED: OnClientRest(); return;
  }
}


