// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//


#include "fi_message"


const string CHAT_COLOR_DEBUG  = MESSAGE_COLOR_DEBUG;
const string CHAT_COLOR_SERVER = MESSAGE_COLOR_SERVER;


// -----------------------------------------------------------------------------

// Shorthand for StringToRGBString
string _RGB(string sString, string sRGB);
string _RGB(string sString, string sRGB)
{
  return StringToRGBString(sString, sRGB);
}


// -----------------------------------------------------------------------------

// Sends a message via SendMessageToPC in Cyan with "Chat" prefixed.
void SendChatMessage(object oPC, string sMessage, string sColor = STRING_COLOR_CYAN);
void SendChatMessage(object oPC, string sMessage, string sColor = STRING_COLOR_CYAN)
{
  SendMessage(oPC, sMessage, sColor, "Chat");
}


// -----------------------------------------------------------------------------

// Get the command payload after the "/action" string.
string GetChatCommandPayload(string sMessage, string sCommand);
string GetChatCommandPayload(string sMessage, string sCommand)
{
  if (GetStringLeft(sCommand, 1) != "/")
    sCommand = "/" + sCommand;

  sMessage = GetSubStringEx(sMessage, GetStringLength(sCommand) + 1);
  sMessage = GetStringTrim(sMessage);
  return sMessage;
}


// -----------------------------------------------------------------------------

// Is there a message left to act upon, or event to log?
int GetIsChatCommandRemaining(object oSpeaker, string sMessage, int nVolume);
int GetIsChatCommandRemaining(object oSpeaker, string sMessage, int nVolume)
{
  // No message left
  if (sMessage == "")
    return FALSE;

  // Internal calls between NPCs
  if (nVolume == TALKVOLUME_SILENT_TALK)
    return FALSE;

  return TRUE;
}


// -----------------------------------------------------------------------------

// Returns TRUE if sMessage is chat command type "/sCommand"
int GetIsChatCommand(string sMessage, string sCommand);
int GetIsChatCommand(string sMessage, string sCommand)
{
  if (!GetStringStartsWith(sMessage, "/"))
     return FALSE;

  sCommand = "/"  + sCommand + " ";

  string sMaybeCommand = GetStringTrim(GetStringLowerCase(sMessage)) + " ";
  sMaybeCommand = GetStringLeft(sMaybeCommand, GetStringLength(sCommand));

  return (sMaybeCommand == sCommand);
}
