// TODO: Review, Test, Document
// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//


#include "fi_console"
#include "fi_const"
#include "fi_object"
#include "fi_string"
#include "fi_time"

#include "x0_i0_match"
#include "x0_inc_generic"


string GetAbilityName(int nAbility)
{
  switch (nAbility)
  {
    case ABILITY_STRENGTH:      return "Strength";     // 135
    case ABILITY_DEXTERITY:     return "Dexterity";    // 133
    case ABILITY_CONSTITUTION:  return "Constitution"; // 132
    case ABILITY_INTELLIGENCE:  return "Intelligence"; // 134
    case ABILITY_WISDOM:        return "Wisdom";       // 136
    case ABILITY_CHARISMA:      return "Charisma";     // 131
  }
  return "";
}


// -----------------------------------------------------------------------------

// Is oObject able to have and keep a conversation?
// cf. GetIsAbleToHear
int GetIsAbleToHaveAConversation(object oObject);
int GetIsAbleToHaveAConversation(object oObject)
{
  if (  FALSE
     || GetIsDead(oObject)
     || GetHasEffect(EFFECT_TYPE_CONFUSED,   oObject)
     || GetHasEffect(EFFECT_TYPE_DOMINATED,  oObject)
     || GetHasEffect(EFFECT_TYPE_PETRIFY,    oObject)
     || GetHasEffect(EFFECT_TYPE_PARALYZE,   oObject)
     || GetHasEffect(EFFECT_TYPE_STUNNED,    oObject)
     || GetHasEffect(EFFECT_TYPE_FRIGHTENED, oObject)
     || GetHasEffect(EFFECT_TYPE_DAZED,      oObject)
     || GetHasEffect(EFFECT_TYPE_SLEEP,      oObject)
     )
    return FALSE;
  return TRUE;
}


// -----------------------------------------------------------------------------

// Is oObject able to hear?
// This is a subsection of GetIsAbleToHaveConversation with EFFECT_TYPE_DEAF in
// addition
int GetIsAbleToHear(object oObject);
int GetIsAbleToHear(object oObject)
{
  if (  FALSE
     || GetIsDead(oObject)
     || GetHasEffect(EFFECT_TYPE_PETRIFY,    oObject)
     || GetHasEffect(EFFECT_TYPE_PARALYZE,   oObject)
     || GetHasEffect(EFFECT_TYPE_DEAF,       oObject)
     )
    return FALSE;
  return TRUE;
}


// -----------------------------------------------------------------------------

// Use local string variable sRoll (default: ROLL) from oObject to roll that
// dice combination, such as 'd3(4)' or 'd6() + d4()'
// Returns amount rolled
int RollDiceFromString(string sRoll = "d6()");
int RollDiceFromString(string sRoll = "d6()")
{
  object oModule = GetModule();

  if (sRoll == "")
    LogWarning("Using default value for RollDiceFromString");
  if (sRoll == "")
    sRoll = "d6()";

  DeleteLocalInt(oModule, RETVAL);
  string sScript = "SetLocalInt(GetModule(), \"RETVAL\", " +  sRoll + ");";
  ExecuteScriptChunk(sScript, oModule);
  int nAmount = GetLocalInt(oModule, RETVAL);
  return nAmount;
}

// void main() {}
