// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//! Handle environment variables

#include "nwnx_util"


// Environment variable that shuts down the server on startup.
const string NWN_SHUTDOWN = "NWN_SHUTDOWN";

// Environment variable used for the debug 'restart' command.
const string NWN_SERVERNAME = "NWN_SERVERNAME";


//! @brief Return TRUE if environment variable sVarName is 1, "YES", "Y", "TRUE"
//! @param sVarName Variable name
//! @param nDefault Default value to return if absent
int GetEnvironmentVariableAsBool(string sVarName, int nDefault = FALSE);
int GetEnvironmentVariableAsBool(string sVarName, int nDefault = FALSE)
{
  string sValue = NWNX_Util_GetEnvironmentVariable(sVarName);
  if (sValue == "")
    return nDefault;
  if (StringToInt(sValue) == 1)
    return TRUE;

  sValue = GetStringUpperCase(sValue);
  if (sValue == "YES")
    return TRUE;
  if (sValue == "Y")
    return TRUE;
  if (sValue == "TRUE")
    return TRUE;
  return FALSE;
}


//! @brief Return value of environment variable sVarName
//! @param sVarName Variable name
//! @param nDefault Default value to return if absent
int GetEnvironmentVariableAsInt(string sVarName, int nDefault = 0);
int GetEnvironmentVariableAsInt(string sVarName, int nDefault = 0)
{
  // We can't simply preprend 0 and read value since
  // NWNX_TWEAKS_STRINGTOINT_BASE_TO_AUTO could be in effect. We could check for
  // the flag, but a simple method for both cases seems better.
  string sValue = NWNX_Util_GetEnvironmentVariable(sVarName);
  if (sValue == "")
    return nDefault;
  return StringToInt(sValue);
}
