// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//

#include "fi_console"
#include "fi_events_name"


// -----------------------------------------------------------------------------

// Expect script to be sExpected for event, warn otherwise.
void ExpectScriptOrWarn(object oObject, int nHandler, string sExpected);
void ExpectScriptOrWarn(object oObject, int nHandler, string sExpected)
{
  string sScript = GetEventScript(oObject, nHandler);
  if (sScript == sExpected)
    return;
  LogWarning("Expected " + GetObjectInfo(oObject) + " script for '" + GetEventScriptName(nHandler) + "' to be " + sExpected);
}

// -----------------------------------------------------------------------------

// Expect script to be sExpected, warn otherwise.
// In all cases, set the script to sExpected
void ExpectAndSetScript(object oObject, int nHandler, string sExpected);
void ExpectAndSetScript(object oObject, int nHandler, string sExpected)
{
  ExpectScriptOrWarn(oObject, nHandler, sExpected);
  SetEventScript(oObject, nHandler,  sExpected);
}


// -----------------------------------------------------------------------------

// Expect script to be sExpected, warn otherwise.
// In all cases, set the script to sExpected
void ExpectAndSetInt(object oObject, string sVarName, int nExpected);
void ExpectAndSetInt(object oObject, string sVarName, int nExpected)
{
  int nValue = GetLocalInt(oObject, sVarName);
  if (nValue == nExpected)
    return;

  LogWarning("Expected " + GetObjectInfo(oObject) + " variable '" + sVarName + "' to be " + IntToString(nExpected));
  SetLocalInt(oObject, sVarName, nExpected);
}

// -----------------------------------------------------------------------------

// Expect tag to be unique.
void ExpectIdenticalTagAndResRef(object oObject);
void ExpectIdenticalTagAndResRef(object oObject)
{
  string sTag = GetTag(oObject);
  string sResRef = GetResRef(oObject);

  if (sTag != sResRef)
    LogFatal("Object " + GetObjectInfo(oObject) + " must have identical resref and tag");
}


// -----------------------------------------------------------------------------
// Expect tag to be unique.
void ExpectUnique(object oObject);
void ExpectUnique(object oObject)
{
  string sTag = GetTag(oObject);
  if (sTag == "")
    return;

  object oOther = GetObjectByTag(sTag, 1);
  if (!GetIsObjectValid(oOther))
    return;

  LogError("Expected " + GetObjectInfo(oObject) + " to have an unique tag. Shared with " + GetObjectInfo(oOther));
}

// -----------------------------------------------------------------------------

// Expect script ressource (without extension) to exist, otherwise, fatal error
void ExpectScriptToExist(string sScript);
void ExpectScriptToExist(string sScript)
{
  // TODO: This calls multiple NWNX_Util_GetFirstResRef and those stop the loading sequence
  string sFound = NWNX_Util_GetFirstResRef(NWNX_UTIL_RESREF_TYPE_NCS, sScript);
  if (sFound != sScript)
    LogFatal("Script '" + sScript + "' not found. Aborting.");
  else
    LogDebug("Script '" + sScript + "' found.");
}

// void main() {}
