// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//

#include "fi_skill"

#include "fi_console"

#include "nwnx_itemprop"

// -----------------------------------------------------------------------------

// Get a detail of item property
string GetItemPropertyDescription(itemproperty oIP);
string GetItemPropertyDescription(itemproperty oIP)
{
  string costValue;
  string paramValue;

  int propId = GetItemPropertyType(oIP);

  string sName = GetStringByStrRef(StringToInt(Get2DAString("itempropdef", "GameStrRef", propId)));

  string sSubType = Get2DAString("itempropdef", "SubTypeResRef", propId);
  if (sSubType != "")
  {
    int nSubTypeId = GetItemPropertySubType(oIP);
    int nStrRef = StringToInt(Get2DAString(sSubType, "Name", nSubTypeId));
    if (nStrRef > 0)
      sSubType = GetStringByStrRef(nStrRef);
  }

  string sCostValue;
  string sCostTableId = Get2DAString("itempropdef", "CostTableResRef", propId);
  if(sCostTableId != "")
  {
    string sCostValueTable = Get2DAString("iprp_costtable", "Name", StringToInt(sCostTableId));
    int nCostId = GetItemPropertyCostTableValue(oIP);
    int nStrRef = StringToInt(Get2DAString(sCostValueTable, "Name", nCostId));
    if(nStrRef > 0)
      sCostValue = GetStringByStrRef(nStrRef);
  }

  string sParamValue;
  string sParamTableId = Get2DAString("itempropdef", "Param1ResRef", propId);
  if (sParamTableId != "")
  {
    string sParamTable = Get2DAString("iprp_paramtable", "TableResRef", StringToInt(sParamTableId));
    int nParamId = GetItemPropertyParam1Value(oIP);
    int nStrRef = StringToInt(Get2DAString(sParamTable, "Name", nParamId));
    if(nStrRef > 0)
      sParamValue = GetStringByStrRef(nStrRef);
  }

  return sName
    + (sSubType != "" ? " " + sSubType : "")
    + (sCostValue != "" ? " " + sCostValue : "")
    + (paramValue != "" ? " " + paramValue : "");
}


// -----------------------------------------------------------------------------

// Get the real gold piece value of oItem.
// Returns 0 if oItem is not a valid item.
int GetItemValue(object oItem);
int GetItemValue(object oItem)
{
  int nPlot = GetPlotFlag(oItem);
  int nStolen = GetStolenFlag(oItem);
  int nCursed = GetItemCursedFlag(oItem);
  int nIdentified = GetIdentified(oItem);

  SetPlotFlag(oItem, FALSE);
  SetIdentified(oItem, TRUE);
  SetStolenFlag(oItem, FALSE);
  SetItemCursedFlag(oItem, FALSE);

  int nValue = GetGoldPieceValue(oItem);

  SetPlotFlag(oItem, nPlot);
  SetStolenFlag(oItem, nStolen);
  SetItemCursedFlag(oItem, nCursed);
  SetIdentified(oItem, nIdentified);
  return nValue;
}


// -----------------------------------------------------------------------------

// Get required lore skill to identify item.
int GetItemRequiredLore(object oItem);
int GetItemRequiredLore(object oItem)
{
  int nValue = GetItemValue(oItem);
  int nLore = 1;
  while (nLore > 0)
  {
    int nMinValue = GetLoreItemMaxValue(nLore - 1);
    int nMaxValue = GetLoreItemMaxValue(nLore);

    if (nValue > nMinValue && nValue <= nMaxValue)
      return nLore;

    nLore += 1;
  }
  return 0;
}


// -----------------------------------------------------------------------------

// Return the itemproperty of type nProperty
itemproperty GetItemPropertyByType(object oItem, int nProperty);
itemproperty GetItemPropertyByType(object oItem, int nProperty)
{
  itemproperty ip = GetFirstItemProperty(oItem);
  while (GetIsItemPropertyValid(ip))
  {
    if (GetItemPropertyType(ip) == nProperty)
      return ip;

    ip = GetNextItemProperty(oItem);
  }
  return ip;
}


// -----------------------------------------------------------------------------

// Get the base AC of a baseitem of type armor.
int GetBaseACValue(object oArmor);
int GetBaseACValue(object oArmor)
{
  // https://nwnlexicon.com/index.php/GetItemACValue
  //
  // Look up in parts_chest.2da the relevant line, which links to the actual AC bonus of the armor
  // We cast it to int, even though the column is technically a float.
  int nAppearance = GetItemAppearance(oArmor, ITEM_APPR_TYPE_ARMOR_MODEL, ITEM_APPR_ARMOR_MODEL_TORSO);
  int nAC = StringToInt(Get2DAString("parts_chest", "ACBONUS", nAppearance));

  return nAC;
}
