// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//! Math functions

// @brief If nValue is less than nMin, returns nMin. If nValue is greater than nMax,
// @brief returns nMax. Otherwise, returns nValue.
int clamp(int nValue, int nMin, int nMax);
int clamp(int nValue, int nMin, int nMax)
{
    return (nValue < nMin) ? nMin : ((nValue > nMax) ? nMax : nValue);
}

// @brief If fValue is less than fMin, returns fMin. If fValue is greater than fMax,
// @brief returns fMax. Otherwise, returns fValue.
float fclamp(float fValue, float fMin, float fMax);
float fclamp(float fValue, float fMin, float fMax)
{
    return (fValue < fMin) ? fMin : ((fValue > fMax) ? fMax : fValue);
}

// @brief Returns the larger of two integers.
int max(int a, int b);
int max(int a, int b)
{
    return (b > a) ? b : a;
}

// @brief Returns the smaller of two integers
int min(int a, int b);
int min(int a, int b)
{
    return (b > a) ? a : b;
}

// @brief Returns the larger of two floats.
float fmax(float a, float b);
float fmax(float a, float b)
{
    return (b > a) ? b : a;
}

// @brief Returns the smaller of two floats.
float fmin(float a, float b);
float fmin(float a, float b)
{
    return (b > a) ? a : b;
}

// @brief Returns f rounded to the nearest whole number.
float floor(float f);
float floor(float f)
{
    return IntToFloat(FloatToInt(f));
}

// @brief Returns the fractional part of f (i.e., the numbers to the right of the
// @brief decimal point).
float frac(float f);
float frac(float f)
{
    return (f - floor(f));
}

// @brief Returns a % b (modulo function).
float fmod(float a, float b);
float fmod(float a, float b)
{
    return a - b * floor(a / b);
}

