// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//

#include "fi_string"

#include "nwnx_util"

#include "x3_inc_string"


// NWN console color codes
const string STRING_COLOR_NONE   = "";
const string STRING_COLOR_GREY   = "444"; // common grey
const string STRING_COLOR_ORANGE = "720";
const string STRING_COLOR_REDISH = "711";
const string STRING_COLOR_YELLOW = "770";
const string STRING_COLOR_COMBAT = "770"; // yellow
const string STRING_COLOR_CYAN   = "077";

// "720"; // Orange
const string MESSAGE_COLOR_DEBUG   = "077"; // Cyan
const string MESSAGE_COLOR_SERVER  = "444"; // Grey
const string MESSAGE_COLOR_SUCCESS = "070"; // Green
const string MESSAGE_COLOR_FAILURE = "711"; // Redish


// -----------------------------------------------------------------------------

// Sends a message via SendMessageToPC with defaults to grey color,
// and with "[Server]" prefixed.
void SendMessage(object oObject, string sMessage, string sColor = STRING_COLOR_GREY, string sCategory = "");
void SendMessage(object oObject, string sMessage, string sColor = STRING_COLOR_GREY, string sCategory = "")
{
  string sLine = "";

  if (GetStringLength(sCategory))
    sCategory = "[" + GetStringTrim(sCategory) + "] ";

  if (GetStringLength(sColor))
    sMessage = StringToRGBString(NWNX_Util_StripColors(sCategory + sMessage), sColor);
  else
    sMessage = NWNX_Util_StripColors(sCategory + sMessage);

  SendMessageToPC(oObject, sMessage);
}


// -----------------------------------------------------------------------------

// Sends a message via SendMessageToPC in Grey with "Server" prefixed.
void SendServerMessage(object oPC, string sMessage, string sColor = MESSAGE_COLOR_SERVER);
void SendServerMessage(object oPC, string sMessage, string sColor = MESSAGE_COLOR_SERVER)
{
  SendMessage(oPC, sMessage, sColor, "Server");
}


// -----------------------------------------------------------------------------

// Sends a message via SendMessageToPC in Green with nothing prefixed.
void SendSuccessMessage(object oPC, string sMessage, string sColor = MESSAGE_COLOR_SUCCESS);
void SendSuccessMessage(object oPC, string sMessage, string sColor = MESSAGE_COLOR_SUCCESS)
{
  SendMessage(oPC, sMessage, sColor);
}


// -----------------------------------------------------------------------------

// Sends a message via SendMessageToPC in Red with nothing prefixed.
void SendFailureMessage(object oPC, string sMessage, string sColor = MESSAGE_COLOR_FAILURE);
void SendFailureMessage(object oPC, string sMessage, string sColor = MESSAGE_COLOR_FAILURE)
{
  SendMessage(oPC, sMessage, sColor);
}

// void main() {}
