// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//


// -----------------------------------------------------------------------------

// When oTarget does an action that removes invisibility
void RemoveInvisibility(object oTarget);
void RemoveInvisibility(object oTarget)
{
  effect eInvis = GetFirstEffect(oTarget);
  while (GetIsEffectValid(eInvis))
  {
    if (GetEffectType(eInvis) == EFFECT_TYPE_INVISIBILITY ||
        GetEffectType(eInvis) == EFFECT_TYPE_IMPROVEDINVISIBILITY )
      RemoveEffect(oTarget, eInvis);
    eInvis = GetNextEffect(oTarget);
  }
}



// -----------------------------------------------------------------------------

// Remove negative effects from oCreature with no VFX
// Except GetEffectType(eBad) == EFFECT_TYPE_CURSE
void RemoveNegativeEffects(object oCreature, int nCurse = FALSE);
void RemoveNegativeEffects(object oCreature, int nCurse = FALSE)
{
  effect eBad = GetFirstEffect(oCreature);
  while (GetIsEffectValid(eBad))
  {
    if (GetEffectType(eBad) == EFFECT_TYPE_ABILITY_DECREASE ||
        GetEffectType(eBad) == EFFECT_TYPE_AC_DECREASE ||
        GetEffectType(eBad) == EFFECT_TYPE_ATTACK_DECREASE ||
        GetEffectType(eBad) == EFFECT_TYPE_DAMAGE_DECREASE ||
        GetEffectType(eBad) == EFFECT_TYPE_DAMAGE_IMMUNITY_DECREASE ||
        GetEffectType(eBad) == EFFECT_TYPE_SAVING_THROW_DECREASE ||
        GetEffectType(eBad) == EFFECT_TYPE_SPELL_RESISTANCE_DECREASE ||
        GetEffectType(eBad) == EFFECT_TYPE_SKILL_DECREASE ||
        GetEffectType(eBad) == EFFECT_TYPE_BLINDNESS ||
        GetEffectType(eBad) == EFFECT_TYPE_DEAF ||
        GetEffectType(eBad) == EFFECT_TYPE_PARALYZE ||
        GetEffectType(eBad) == EFFECT_TYPE_NEGATIVELEVEL ||
        GetEffectType(eBad) == EFFECT_TYPE_FRIGHTENED ||
        GetEffectType(eBad) == EFFECT_TYPE_DAZED ||
        GetEffectType(eBad) == EFFECT_TYPE_CONFUSED ||
        GetEffectType(eBad) == EFFECT_TYPE_POISON ||
        GetEffectType(eBad) == EFFECT_TYPE_DISEASE ||
        (nCurse && GetEffectType(eBad) == EFFECT_TYPE_CURSE )
        )
      RemoveEffect(oCreature, eBad);
    eBad = GetNextEffect(oCreature);
  }
}


// -----------------------------------------------------------------------------

// Raise the dead, unless if undead.
// Return TRUE if raised
int RaiseDead(object oTarget);
int RaiseDead(object oTarget)
{
  if (!GetIsObjectValid(oTarget))
    return FALSE;

  // Declare major variables
  effect eRaise = EffectResurrection();
  effect eVis = EffectVisualEffect(VFX_IMP_RAISE_DEAD);

  // Fire cast spell at event for the specified target
  SignalEvent(oTarget, EventSpellCastAt(OBJECT_SELF, SPELL_RAISE_DEAD, FALSE));
  if (!GetIsDead(oTarget))
    return FALSE;

  int nRace = GetRacialType(oTarget);
  if (nRace == RACIAL_TYPE_UNDEAD)
    return FALSE;

  // Apply raise dead effect and VFX impact
  ApplyEffectAtLocation(DURATION_TYPE_INSTANT, eVis, GetLocation(oTarget));
  ApplyEffectToObject(DURATION_TYPE_INSTANT, eRaise, oTarget);
  return TRUE;
}


// -----------------------------------------------------------------------------

// Raise the dead, unless if undead. And heal on top.
// Return TRUE if raised
int Ressurection(object oTarget);
int Ressurection(object oTarget)
{
  if (!GetIsObjectValid(oTarget))
    return FALSE;

  SignalEvent(oTarget, EventSpellCastAt(OBJECT_SELF, SPELL_RESURRECTION, FALSE));
  if (!GetIsDead(oTarget))
    return FALSE;

  int nRace = GetRacialType(oTarget);
  if (nRace == RACIAL_TYPE_UNDEAD)
    return FALSE;

  effect eRaise = EffectResurrection();
  effect eHeal = EffectHeal(GetMaxHitPoints(oTarget) + 10);
  effect eVis = EffectVisualEffect(VFX_IMP_RAISE_DEAD);

  ApplyEffectToObject(DURATION_TYPE_INSTANT, eRaise, oTarget);
  ApplyEffectToObject(DURATION_TYPE_INSTANT, eHeal, oTarget);
  ApplyEffectAtLocation(DURATION_TYPE_INSTANT, eVis, GetLocation(oTarget));
  return TRUE;
}


void ApplyIncorporeal(object oTarget, int nPercentage);
void ApplyIncorporeal(object oTarget, int nPercentage)
{
  effect eConceal = EffectConcealment(nPercentage, MISS_CHANCE_TYPE_NORMAL);
  eConceal = ExtraordinaryEffect(eConceal);

  effect eGhost = EffectCutsceneGhost();
  eGhost = ExtraordinaryEffect(eGhost);

  ApplyEffectToObject(DURATION_TYPE_PERMANENT, eConceal, oTarget);
  ApplyEffectToObject(DURATION_TYPE_PERMANENT, eGhost, oTarget);
}

// void main() {}
