// =============================================================================
// Framework include
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//

#include "fi_console"
#include "fi_string"

#include "nwnx_util"

// -----------------------------------------------------------------------------
// Limit the script to 10% of the Instruction Limit
int InstructionLimitPassed(string sPrefix);
int InstructionLimitPassed(string sPrefix)
{
  float fPct = 100.0f * IntToFloat(NWNX_Util_GetInstructionsExecuted()) / IntToFloat(NWNX_Util_GetInstructionLimit());
  if (fPct >= 10.0f)
  {
    LogWarning(sPrefix + ": Ten percent instruction limit encountered. Aborting current script. "
      + "Instructions: " + IntToString(NWNX_Util_GetInstructionsExecuted()), CONSOLE_COLOR_RED);
    return TRUE;
  }
  return FALSE;
}


// -----------------------------------------------------------------------------
// Returns a tag. This tag will not be associated with any object.
// It will be 32 in length
string GetRandomTag();
string GetRandomTag()
{
  string sTag = GetRandomUUID();
  sTag = GetStringReplace(sTag, "-", "A");
  return sTag;
}


// -----------------------------------------------------------------------------
// Returns a tag. This tag will not be associated with any object.
// It will be 32 in length
string SetUniqueTag(object oObject);
string SetUniqueTag(object oObject)
{
  string sTag = GetTag(oObject);
  if (GetStringLength(sTag) == 36)
    return sTag;

  string sNewTag = GetRandomTag();
  SetTag(oObject, sNewTag);
  return sNewTag;
}


// -----------------------------------------------------------------------------
string LocationToString(location locLocation)
{
  string sAreaTag  = GetTag(GetAreaFromLocation(locLocation));
  vector vPosition = GetPositionFromLocation(locLocation);
  float  fFacing   = GetFacingFromLocation(locLocation);

  return sAreaTag +
         "#" + FloatToString(vPosition.x, 0, 5) + // x
         "#" + FloatToString(vPosition.y, 0, 5) + // y
         "#" + FloatToString(vPosition.z, 0, 5) + // z
         "#" + FloatToString(fFacing, 0, 5) +     // facing
         "#";
}


// void main() {}
