// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//

/*! rst
Seasons. Sets dawn and dusk hours according to month.
*/


#include "fi_events"
#include "fi_console"


// -----------------------------------------------------------------------------
void OnDay();
void OnDay()
{
  string sLog = "A new day starts: " +       IntToString(GetCalendarYear())
                                     + "-" + IntToString(GetCalendarMonth())
                                     + "-" + IntToString(GetCalendarDay())
                                     + ". Season is ";

  int nCalendarMonth = GetCalendarMonth();
  switch (nCalendarMonth)
  {
    // Spring (Months 3-5): Dawn at 0700, Dusk at 1900
    case 3:
    case 4:
    case 5:
      NWNX_Util_SetDawnHour(7);
      NWNX_Util_SetDuskHour(19);
      LogInfo(sLog + "spring.");
      break;
    // Summer (Months 6-8): Dawn at 0600, Dusk at 2000
    case 6:
    case 7:
    case 8:
      NWNX_Util_SetDawnHour(6);
      NWNX_Util_SetDuskHour(20);
      LogInfo(sLog + "summer.");
      break;
    // Autumn (Months 9-11): Dawn at 0700, Dusk at 1900
    case 9:
    case 10:
    case 11:
      NWNX_Util_SetDawnHour(7);
      NWNX_Util_SetDuskHour(19);
      LogInfo(sLog + "autumn.");
      break;
    // Winter (Months 12-2): Dawn at 0800, Dusk at 1800
    case 12:
    case 1:
    case 2:
      NWNX_Util_SetDawnHour(8);
      NWNX_Util_SetDuskHour(18);
      LogInfo(sLog + "winter.");
      break;
  }
}


// =============================================================================
void main()
{
  object oPC;
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      OnDay();
      RegisterEvent(EVENT_CALENDAR_DAY);
      return;

    case EVENT_CALENDAR_DAY: OnDay(); return;

    case EVENT_INFO_CONSOLE: // TODO
  }
}


