// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Persistent time, saved every minute in database.
*/


#include "fi_events"
#include "fi_campaign"
#include "fi_console"


// -----------------------------------------------------------------------------
void SaveCalendar();
void SaveCalendar()
{
  string sCampaign = GetCampaignName("time");

  SetCampaignInt(sCampaign, "YEAR",   GetCalendarYear());
  SetCampaignInt(sCampaign, "MONTH",  GetCalendarMonth());
  SetCampaignInt(sCampaign, "DAY",    GetCalendarDay());
  SetCampaignInt(sCampaign, "HOUR",   GetTimeHour());
  SetCampaignInt(sCampaign, "MINUTE", GetTimeMinute());
  SetCampaignInt(sCampaign, "SECOND", GetTimeSecond());
}


// -----------------------------------------------------------------------------
void LoadCalendar();
void LoadCalendar()
{
  string sCampaign = GetCampaignName("time");

  int nYear    = GetCampaignInt(sCampaign, "YEAR");
  int nMonth   = GetCampaignInt(sCampaign, "MONTH");
  int nDay     = GetCampaignInt(sCampaign, "DAY");
  int nHour    = GetCampaignInt(sCampaign, "HOUR");
  int nMinute  = GetCampaignInt(sCampaign, "MINUTE");
  int nSeconds = GetCampaignInt(sCampaign, "SECOND");

  if (nYear == 0)
  {
    LogWarning("Persistent calendar is not set");
    SaveCalendar();
  }
  else
  {
    LogInfo("Persistent calendar set to " + IntToString(nYear) + "-" + IntToString(nMonth) + "-" + IntToString(nDay));
    SetCalendar(nYear, nMonth, nDay);
    SetTime(nHour, nMinute, nSeconds, 0);
  }
}


// =============================================================================
void main()
{
  object oPC;
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      LoadCalendar();
      RegisterEvent(EVENT_MODULE_REALTIME_MINUTE);
      return;

    case EVENT_MODULE_REALTIME_MINUTE: SaveCalendar(); return;

    case EVENT_INFO_CONSOLE: // TODO
  }
}


