// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Enables chat commands for various purposes.
*/


#include "fi_core"
#include "fi_chat"
#include "fi_events"


const string C1 = CHAT_COLOR_DEBUG;
const string C2 = CHAT_COLOR_SERVER;

// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oSpeaker = GetPCChatSpeaker();

  SendMessageToPC(oSpeaker, _RGB("/area: ", C1) + _RGB("Get area name.", C2));
}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oModule = GetModule();
  object oSpeaker = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();

  string sCommand = GetStringTrim(GetStringLowerCase(sMessage));

  if (GetIsChatCommand(sCommand, "area"))
  {
    object oArea = GetArea(oSpeaker);
    string sArea = GetName(oArea);
    if (GetIsDebug(oSpeaker))
      SendChatMessage(oSpeaker, "Area is '" + sArea + "' (" + GetTag(oArea) + ").");
    else
      SendChatMessage(oSpeaker, "Area is '" + sArea + "'.");
    SetPCChatMessage("");
    return;
  }
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_CLIENT_ENTER);
      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);
      return;

    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}
