// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Enables standard actions via chat commands.

Enables ``/export`` to export PC to servervault.
Enables ``/rest`` to rest.
*/


#include "fi_core"
#include "fi_chat"
#include "fi_events"


const string C1 = CHAT_COLOR_DEBUG;
const string C2 = CHAT_COLOR_SERVER;

// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oSpeaker = GetPCChatSpeaker();

  SendMessageToPC(oSpeaker, _RGB("/export: ", C1) + _RGB("Force save the PC to servervault.", C2));
  SendMessageToPC(oSpeaker, _RGB("/rest: ", C1) + _RGB("Rest", C2));
}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oModule = GetModule();
  object oSpeaker = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();

  string sCommand = GetStringTrim(GetStringLowerCase(sMessage));

  if (GetIsChatCommand(sCommand, "export"))
  {
    ExportSingleCharacter(oSpeaker);
    SendChatMessage(oSpeaker, "Player has been saved to servervault.");
    SetPCChatMessage("");
    return;
  }

  if (GetIsChatCommand(sCommand, "rest"))
  {
    AssignCommand(oSpeaker, ActionRest(TRUE));
    SetPCChatMessage("");
    return;
  }
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_CLIENT_ENTER);
      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);
      return;

    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}
