// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

//! Add debug only chat commands

#include "fi_action"
#include "fi_chat"
#include "fi_events"

#include "fi_pc"


// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oSpeaker = GetPCChatSpeaker();
  string C1 = CHAT_COLOR_DEBUG;
  string C2 = CHAT_COLOR_SERVER;

  if (!GetIsDebug(oSpeaker))
    return;

  SendMessageToPC(oSpeaker, _RGB("/exec STRING: ", C1) + _RGB("Execute script", C2));
  SendMessageToPC(oSpeaker, _RGB("/give STRING/INT: ", C1) + _RGB("Create item on PC, or give *integer* gold.", C2));
  SendMessageToPC(oSpeaker, _RGB("/givexp INT: ", C1) + _RGB("Give *integer* gold.", C2));
  SendMessageToPC(oSpeaker, _RGB("/haste: ", C1) + _RGB("Execute haste", C2));
  SendMessageToPC(oSpeaker, _RGB("/heal: ", C1) + _RGB("Heal self", C2));
  SendMessageToPC(oSpeaker, _RGB("/identify: ", C1) + _RGB("Execute identify all", C2));
  SendMessageToPC(oSpeaker, _RGB("/info: ", C1) + _RGB("Log unit info", C2));
  SendMessageToPC(oSpeaker, _RGB("/level INT: ", C1) + _RGB("Set PC level to INT", C2));
  SendMessageToPC(oSpeaker, _RGB("/spawn STRING: ", C1) + _RGB("Spawn an object.", C2));
  SendMessageToPC(oSpeaker, _RGB("/time INT: ", C1) + _RGB("Add or remove hours.", C2));
}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oModule = GetModule();
  object oSpeaker = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();
  int    nVolume  = GetPCChatVolume();

  if (GetIsChatCommand(sMessage, "debug"))
  {
    string sPassword = GetChatCommandPayload(sMessage, "debug");
    if (sPassword == NWNX_Administration_GetDMPassword())
    {
      SetLocalInt(oSpeaker, DEBUG, 1);
      SendChatMessage(oSpeaker, "success: PC is now in debug mode");
    }
    SetPCChatMessage("");
    return;
  }

  if (!(GetIsDebug(oSpeaker)))
    return;

  string sCommand = GetStringTrim(sMessage);
  if (GetIsChatCommand(sCommand, "exec"))
  {
    string sScript = GetChatCommandPayload(sCommand, "exec");
    ExecuteScript(sScript, oSpeaker);
    SetPCChatMessage("");
    return;
  }

  if (GetIsChatCommand(sCommand, "give"))
  {

    string sResRef = GetChatCommandPayload(sCommand, "give");
    string sInt = IntToString(StringToInt(sResRef));

    object oItem = CreateItemOnObject(sResRef, oSpeaker);
    if (!GetIsObjectValid(oItem))
      GiveGoldToCreature(oSpeaker, StringToInt(sResRef));

    SetPCChatMessage("");
    return;
  }

  if (GetIsChatCommand(sCommand, "givexp"))
  {
    string sResRef = GetChatCommandPayload(sCommand, "givexp");
    string sInt = IntToString(StringToInt(sResRef));
    if (sInt != "")
      GiveXPToCreature(oSpeaker, StringToInt(sResRef));

    SetPCChatMessage("");
    return;
  }

  if (GetIsChatCommand(sCommand, "haste"))
  {
    effect eHaste = SupernaturalEffect(EffectHaste());
    ApplyEffectToObject(DURATION_TYPE_PERMANENT, eHaste, oSpeaker);
    SetPCChatMessage("");
    return;
  }

  if (GetIsChatCommand(sCommand, "heal"))
  {
    effect eRaise = EffectResurrection();
    effect eHeal = EffectHeal(GetMaxHitPoints(oSpeaker) + 10);
    effect eVis = EffectVisualEffect(VFX_IMP_RAISE_DEAD);

    ApplyEffectToObject(DURATION_TYPE_INSTANT, eRaise, oSpeaker);
    ApplyEffectToObject(DURATION_TYPE_INSTANT, eHeal, oSpeaker);
    ApplyEffectAtLocation(DURATION_TYPE_INSTANT, eVis, GetLocation(oSpeaker));

    ActionTempleHeal(oSpeaker);

    SetPCChatMessage("");
    return;
  }

  if (GetIsChatCommand(sCommand, "identify"))
  {
    object oItem = GetFirstItemInInventory(oSpeaker);
    while (GetIsObjectValid(oItem))
    {
      SetIdentified(oItem, TRUE);
      oItem = GetNextItemInInventory(oSpeaker);
    }
    SetPCChatMessage("");
    return;
  }

  if (GetIsChatCommand(sCommand, "info"))
  {
    BroadcastEvent(EVENT_INFO_CONSOLE);
    SetPCChatMessage("");
    return;
  }

  if (GetIsChatCommand(sCommand, "level"))
  {
    string sLevel = GetChatCommandPayload(sCommand, "level");
    int nLevel = StringToInt(sLevel);
    if (nLevel == 0 || nLevel < 0 || nLevel > 40)
      SendChatMessage(oSpeaker, "error: '" + sLevel + "' is not a valid number.");
    else
      SetXP(oSpeaker, GetXPForLevel(nLevel));

    SetPCChatMessage("");
    return;
  }


  if (GetIsChatCommand(sCommand, "spawn"))
  {
    string sTemplate = GetChatCommandPayload(sCommand, "spawn");

    CreateObject(OBJECT_TYPE_CREATURE, sTemplate, GetLocation(oSpeaker));

    SetPCChatMessage("");
    return;
  }


  if (GetIsChatCommand(sCommand, "time"))
  {
    string sResRef = GetChatCommandPayload(sCommand, "time");
    string sInt = IntToString(StringToInt(sResRef));
    int nInt = StringToInt(sInt);

    int nHour = GetTimeHour();
    int nMinute  = GetTimeMinute();
    int nSeconds = GetTimeSecond();
    if (sInt != "" && nInt > 0)
    {
      nHour = nHour + nInt;
      // To avoid trying to set back time (and thus leap forward a day!)
      nSeconds += 1;
      SetTime(nHour, nMinute, nSeconds, 0);
    }

    int nYear    = GetCalendarYear();
    int nMonth   = GetCalendarMonth();
    int nDay     = GetCalendarDay();
        nHour    = GetTimeHour();
        nMinute  = GetTimeMinute();
        nSeconds = GetTimeSecond();

    string sMessage = "Current date/time: " +       IntToString(nYear)
                                            + "-" + IntToString(nMonth)
                                            + "-" + IntToString(nDay)
                                            + " " + GetStringLPad(IntToString(nHour), 2, "0")
                                            + ":" + GetStringLPad(IntToString(nMinute), 2, "0")
                                            + ":" + GetStringLPad(IntToString(nSeconds), 2, "0");

    SendMessage(oSpeaker, sMessage, MESSAGE_COLOR_SUCCESS);
    LogInfo(sMessage);

    SetPCChatMessage("");
    return;
  }
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);
      return;

    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}
