// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Enables the player to delete the PC.

Enables ``/delete`` command with confirmation.
*/


#include "fi_core"
#include "fi_chat"
#include "fi_events"


const string C1 = CHAT_COLOR_DEBUG;
const string C2 = CHAT_COLOR_SERVER;

// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oSpeaker = GetPCChatSpeaker();

  if (!GetIsPossessedFamiliar(oSpeaker))
    SendMessageToPC(oSpeaker, _RGB("/delete: ", C1) + _RGB("Delete character.", C2));
}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oModule = GetModule();
  object oSpeaker = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();

  string sCommand = GetStringTrim(GetStringLowerCase(sMessage));

  if (GetIsChatCommand(sCommand, "delete") && !GetIsPossessedFamiliar(oSpeaker))
  {
    string sPasswordIn   = GetStringUpperCase(GetChatCommandPayload(sCommand, "delete"));
    string sPasswordChar = GetLocalString(oSpeaker, "DELETE_PASSWORD");
    if ((sPasswordIn == sPasswordChar) && (sPasswordIn != ""))
    {
      string sMessage = "Player character has been deleted.";
      LogWarning(GetObjectInfo(oSpeaker) + " decided to delete this character.");
      NWNX_Administration_DeleteTURD(GetPCPlayerName(oSpeaker), GetName(oSpeaker));
      NWNX_Administration_DeletePlayerCharacter(oSpeaker, TRUE, sMessage);
      BootPC(oSpeaker, sMessage);
    }
    else
    {
      sPasswordChar = GetStringUpperCase(GetStringLeft(GetRandomUUID(), 4));
      SetLocalString(oSpeaker, "DELETE_PASSWORD", sPasswordChar);
      SendServerMessage(oSpeaker, "To confirm the deletion of your character, please type '/delete " + sPasswordChar + "'.", STRING_COLOR_REDISH);
    }
    SetPCChatMessage("");
    return;
  }
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_CLIENT_ENTER);
      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);
      return;

    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}
