// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Allow emotes from chat commands.

Emotes will be available via ``/emote``, ``/e`, or the action itself ``/bow``. Chat ``/e help`` for help.
*/



#include "fi_chat"
#include "fi_events"


const float EMOTE_DURATION_LOOPING = 86400.0f;


// -----------------------------------------------------------------------------
void DoEmote(object oPlayer, int nEmote, float fDuration = 0.0f)
{
  AssignCommand(oPlayer, ClearAllActions());
  AssignCommand(oPlayer, ActionPlayAnimation(nEmote, 1.0, fDuration));
}


// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oSpeaker = GetPCChatSpeaker();
  string C1 = CHAT_COLOR_DEBUG;
  string C2 = CHAT_COLOR_SERVER;

  SendMessageToPC(oSpeaker, _RGB("/e EMOTE: ", C1) + _RGB("Play an emote. '/e help' for list of emotes.", C2));
}


// -----------------------------------------------------------------------------
int OnEmote(string sEmote);
int OnEmote(string sEmote)
{
  object oPlayer = GetPCChatSpeaker();
  string C1 = CHAT_COLOR_DEBUG;
  string C2 = CHAT_COLOR_SERVER;

  sEmote = GetStringTrim(sEmote);
  sEmote = GetStringLowerCase(sEmote);

  if (sEmote == "help")
    SendMessageToPC(oPlayer, _RGB("Available Emotes: bow, duck, dodge, drink, greet, bored, scratch, read, salute, steal, taunt, victory (victory1, victory2, victory3), cast (cast1, cast2), deadback, deadfront, low, mid, meditate, drunk, tired, sit, spasm, forceful, laugh, talk, plead, worship", C2));
  else if (sEmote == "bow")       DoEmote(oPlayer, ANIMATION_FIREFORGET_BOW);
  else if (sEmote == "duck")      DoEmote(oPlayer, ANIMATION_FIREFORGET_DODGE_DUCK);
  else if (sEmote == "dodge")     DoEmote(oPlayer, ANIMATION_FIREFORGET_DODGE_SIDE);
  else if (sEmote == "drink")     DoEmote(oPlayer, ANIMATION_FIREFORGET_DRINK);
  else if (sEmote == "greet")     DoEmote(oPlayer, ANIMATION_FIREFORGET_GREETING);
  else if (sEmote == "bored")     DoEmote(oPlayer, ANIMATION_FIREFORGET_PAUSE_BORED);
  else if (sEmote == "scratch")   DoEmote(oPlayer, ANIMATION_FIREFORGET_PAUSE_SCRATCH_HEAD);
  else if (sEmote == "read")      DoEmote(oPlayer, ANIMATION_FIREFORGET_READ);
  else if (sEmote == "salute")    DoEmote(oPlayer, ANIMATION_FIREFORGET_SALUTE);
  else if (sEmote == "steal")     DoEmote(oPlayer, ANIMATION_FIREFORGET_STEAL);
  else if (sEmote == "taunt")     DoEmote(oPlayer, ANIMATION_FIREFORGET_TAUNT);
  else if (sEmote == "victory")   DoEmote(oPlayer, ANIMATION_FIREFORGET_VICTORY1);
  else if (sEmote == "victory1")  DoEmote(oPlayer, ANIMATION_FIREFORGET_VICTORY1);
  else if (sEmote == "victory2")  DoEmote(oPlayer, ANIMATION_FIREFORGET_VICTORY2);
  else if (sEmote == "victory3")  DoEmote(oPlayer, ANIMATION_FIREFORGET_VICTORY3);
  else if (sEmote == "cast")      DoEmote(oPlayer, ANIMATION_LOOPING_CONJURE1, EMOTE_DURATION_LOOPING);
  else if (sEmote == "cast1")     DoEmote(oPlayer, ANIMATION_LOOPING_CONJURE1, EMOTE_DURATION_LOOPING);
  else if (sEmote == "cast2")     DoEmote(oPlayer, ANIMATION_LOOPING_CONJURE2, EMOTE_DURATION_LOOPING);
  else if (sEmote == "deadback")  DoEmote(oPlayer, ANIMATION_LOOPING_DEAD_BACK, EMOTE_DURATION_LOOPING);
  else if (sEmote == "deadfront") DoEmote(oPlayer, ANIMATION_LOOPING_DEAD_FRONT, EMOTE_DURATION_LOOPING);
  else if (sEmote == "low")       DoEmote(oPlayer, ANIMATION_LOOPING_GET_LOW, EMOTE_DURATION_LOOPING);
  else if (sEmote == "mid")       DoEmote(oPlayer, ANIMATION_LOOPING_GET_MID, EMOTE_DURATION_LOOPING);
  else if (sEmote == "meditate")  DoEmote(oPlayer, ANIMATION_LOOPING_MEDITATE, EMOTE_DURATION_LOOPING);
  else if (sEmote == "drunk")     DoEmote(oPlayer, ANIMATION_LOOPING_PAUSE_DRUNK, EMOTE_DURATION_LOOPING);
  else if (sEmote == "tired")     DoEmote(oPlayer, ANIMATION_LOOPING_PAUSE_TIRED, EMOTE_DURATION_LOOPING);
  else if (sEmote == "sit")       DoEmote(oPlayer, ANIMATION_LOOPING_SIT_CROSS, EMOTE_DURATION_LOOPING);
  else if (sEmote == "spasm")     DoEmote(oPlayer, ANIMATION_LOOPING_SPASM, EMOTE_DURATION_LOOPING);
  else if (sEmote == "forceful")  DoEmote(oPlayer, ANIMATION_LOOPING_TALK_FORCEFUL, EMOTE_DURATION_LOOPING);
  else if (sEmote == "laugh")     DoEmote(oPlayer, ANIMATION_LOOPING_TALK_LAUGHING, EMOTE_DURATION_LOOPING);
  else if (sEmote == "talk")      DoEmote(oPlayer, ANIMATION_LOOPING_TALK_NORMAL, EMOTE_DURATION_LOOPING);
  else if (sEmote == "plead")     DoEmote(oPlayer, ANIMATION_LOOPING_TALK_PLEADING, EMOTE_DURATION_LOOPING);
  else if (sEmote == "worship")   DoEmote(oPlayer, ANIMATION_LOOPING_WORSHIP, EMOTE_DURATION_LOOPING);
  else
    return FALSE;
  return TRUE;
}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oModule = GetModule();
  object oSpeaker = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();

  string sCommand = GetStringTrim(GetStringLowerCase(sMessage));

  // Catch "/e bow", ...
  if (GetIsChatCommand(sCommand, "e"))
  {
    string sEmote = GetChatCommandPayload(sCommand, "e");
    OnEmote(sEmote);
    SetPCChatMessage("");
    return;
  }

  // Catch "/emote bow", ...
  if (GetIsChatCommand(sCommand, "emote"))
  {
    string sEmote = GetChatCommandPayload(sCommand, "emote");
    OnEmote(sEmote);
    SetPCChatMessage("");
    return;
  }

  // Catch "/bow", ...
  if (GetStringLeft(sCommand, 1) == "/" && (sCommand != "/help"))
  {
    string sEmote = GetSubStringEx(sCommand, 1);
    if (OnEmote(sEmote))
      SetPCChatMessage("");
    return;
  }
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);
      return;

    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}
