// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

//! Add chat commands for PC options via ``/option`` (such as turning off cloak)

// TODO: Make persistent.

#include "fi_chat"
#include "fi_events"



// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oSpeaker = GetPCChatSpeaker();
  string C1 = CHAT_COLOR_DEBUG;
  string C2 = CHAT_COLOR_SERVER;

  SendMessageToPC(oSpeaker, _RGB("/o HELP: ", C1) + _RGB("Turn options on and off.", C2));
}


// -----------------------------------------------------------------------------

void ToggleHelmet(object oPC);
void ToggleHelmet(object oPC)
{
  object oItem = GetItemInSlot(INVENTORY_SLOT_HEAD, oPC);
  if (!GetIsObjectValid(oItem))
    return;

  int nShown = GetHiddenWhenEquipped(oItem);
  SetHiddenWhenEquipped(oItem, !nShown);
}


// -----------------------------------------------------------------------------

void ToggleCloak(object oPC);
void ToggleCloak(object oPC)
{
  object oItem = GetItemInSlot(INVENTORY_SLOT_CLOAK, oPC);
  if (!GetIsObjectValid(oItem))
    return;

  int nShown = GetHiddenWhenEquipped(oItem);
  SetHiddenWhenEquipped(oItem, !nShown);
}


// -----------------------------------------------------------------------------
int OnOption(object oPC, string sCommand);
int OnOption(object oPC, string sCommand)
{
  string C1 = CHAT_COLOR_DEBUG;
  string C2 = CHAT_COLOR_SERVER;

  sCommand = GetStringTrim(sCommand);
  sCommand = GetStringLowerCase(sCommand);

  string sOption = StringParse(sCommand);
  string sValue  = StringRemoveParsed(sCommand, sOption);

  if (sOption == "help")
    SendMessageToPC(oPC, _RGB("Available Options: cloak, helmet", C2));
  else if (sOption == "cloak")
    ToggleCloak(oPC);
  else if (sOption == "helmet")
    ToggleHelmet(oPC);
  else
    return FALSE;
  return TRUE;
}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oPC = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();

  string sCommand = GetStringTrim(GetStringLowerCase(sMessage));

  // Catch "/option cloak", ...
  if (GetIsChatCommand(sCommand, "option"))
  {
    string sOption = GetChatCommandPayload(sCommand, "option");
    OnOption(oPC, sOption);
    SetPCChatMessage("");
  }
}


// -----------------------------------------------------------------------------
void OnEnter();
void OnEnter()
{
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_CLIENT_ENTER);
      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);
      return;

    case EVENT_CLIENT_ENTER: OnEnter(); return;
    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}

