// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Add a ``/restart`` command for admins and debug.
*/


#include "fi_chat"
#include "fi_events"

#include "fi_pc"


// -----------------------------------------------------------------------------
void OnHelp();
void OnHelp()
{
  object oPC = GetPCChatSpeaker();
  string C1 = CHAT_COLOR_DEBUG;
  string C2 = CHAT_COLOR_SERVER;

  if (!GetIsDebug(oPC) || GetIsDM(oPC) || GetIsPlayerDM(oPC))
    return;

  SendMessageToPC(oPC, _RGB("/restart: ", C1) + _RGB("Restart the server", C2));
}


// -----------------------------------------------------------------------------
void OnChat();
void OnChat()
{
  object oModule = GetModule();
  object oPC = GetPCChatSpeaker();
  string sMessage = GetPCChatMessage();

  if (!GetIsDebug(oPC) || GetIsDM(oPC) || GetIsPlayerDM(oPC))
    return;

  string sCommand = GetStringTrim(sMessage);
  if (GetIsChatCommand(sCommand, "restart"))
  {
    string sFileName = NWNX_Util_GetModuleFile();
    LogWarning("Restart command issued by " + GetObjectInfo(oPC));
    SendMessage(oPC, "Server will restart '" + sFileName + ".mod'", MESSAGE_COLOR_SUCCESS);
    DelayCommand(0.0f, StartNewModule(sFileName));
    ExportAllCharacters();
    SetPCChatMessage("");
    return;
  }
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_PLAYER_CHAT);
      RegisterEvent(EVENT_PLAYER_CHAT_HELP);
      return;

    case EVENT_PLAYER_CHAT: OnChat(); return;
    case EVENT_PLAYER_CHAT_HELP: OnHelp(); return;
  }
}
