// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
// https://nwn.wiki/display/NWN1/Journal+Editor
//
// TODO: Basic functionnality works, but when we login, we get the quest complete sound.
//
// gui_journaladd.wav -- Triggered by AddJournalQuestEntry when no quest exists yet
// gui_quest_done.wav -- Triggered by AddJournalQuestEntry when a quest is "done" or "complete" - whether failed or succeeded

/*! rst
Persistent journal.

When using the dialog editor to edit the journal, it is recommended to set :ref:`fw_dlg_act_journ` as the dialog action.
*/


#include "fi_console"
#include "fi_journal"
#include "fi_events"
#include "fi_string"


// -----------------------------------------------------------------------------
void LoadJournal(object oPC);
void LoadJournal(object oPC)
{
  if (!GetIsPC(oPC))
    return;

  LogDebug("Journal: Loading...");

  int i;
  while (i >= 0)
  {
    string szi = IntToString(i++);

    string sQuest = NWNX_Object_GetString(oPC, NW_JOURNAL_ENTRY + szi + "_NAME");
    if (sQuest == "")
    {
      i = -1;
      continue;
    }

    SetLocalInt(oPC, NW_JOURNAL_ENTRY + sQuest, NWNX_Object_GetInt(oPC, NW_JOURNAL_ENTRY + szi));
    SetLocalInt(oPC, NW_JOURNAL_DATE + sQuest,  NWNX_Object_GetInt(oPC, NW_JOURNAL_DATE + szi));
    SetLocalInt(oPC, NW_JOURNAL_TIME + sQuest,  NWNX_Object_GetInt(oPC, NW_JOURNAL_TIME + szi));
  }
  LogDebug("Journal: Loaded.");
}


// -----------------------------------------------------------------------------
void SaveJournal(object oPC);
void SaveJournal(object oPC)
{
  if (!GetIsPC(oPC))
    return;

  LogDebug("Journal: Saving...");

  int nIndex = 0;
  int nCount = NWNX_Object_GetLocalVariableCount(oPC);
  NWNX_Object_DeleteVarRegex(oPC, "NW_JOURNAL_ENTRY\*");

  int i;
  for (i = 0; i <= nCount; i++)
  {
    struct NWNX_Object_LocalVariable stVar =  NWNX_Object_GetLocalVariable(oPC, i);
    if (stVar.type = 1 && GetStringStartsWith(stVar.key, NW_JOURNAL_ENTRY))
    {
      string sQuest = StringRemoveParsed(stVar.key, NW_JOURNAL_ENTRY);

      string szi = IntToString(nIndex++);
      NWNX_Object_SetString(oPC, NW_JOURNAL_ENTRY + szi + "_NAME", sQuest, TRUE);
      NWNX_Object_SetInt(oPC, NW_JOURNAL_ENTRY + szi, GetLocalInt(oPC, NW_JOURNAL_ENTRY + sQuest), TRUE);
      NWNX_Object_SetInt(oPC, NW_JOURNAL_DATE  + szi, GetLocalInt(oPC, NW_JOURNAL_DATE + sQuest),  TRUE);
      NWNX_Object_SetInt(oPC, NW_JOURNAL_TIME  + szi, GetLocalInt(oPC, NW_JOURNAL_TIME + sQuest),  TRUE);
    }
  }

  LogDebug("Journal: Saved...");
}


 // =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      // Areas can be recreated, thus invalidating the map pins area.
      RegisterEvent(EVENT_CLIENT_ENTER_ONCE);
      RegisterEvent(EVENT_CLIENT_DISCONNECT_BEFORE);
      RegisterEvent(EVENT_JOURNAL_UPDATED);
      return;

    case EVENT_CLIENT_ENTER_ONCE:
      LoadJournal(GetEnteringObject());
      return;

    case EVENT_CLIENT_DISCONNECT_BEFORE:
    case EVENT_JOURNAL_UPDATED:
      SaveJournal(OBJECT_SELF);;
      return;
  }
}

