// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
// Player map pins are local variables stored on the PC object.
//
//     NW_TOTAL_MAP_PINS - Int of number of map pins player has placed
//     NW_MAP_PIN_NTRY_# - String of map pin text
//     NW_MAP_PIN_XPOS_# - Float of X coordinate
//     NW_MAP_PIN_YPOS_# - Float of Y coordinate
//     NW_MAP_PIN_AREA_# - Object of area
//
// TODO: Basic functionnality works, but if we destroy an area, not recreate it,
// then save the pins, that area's map pins will disappear.
// TODO: Rewrite

#include "fi_core"
#include "fi_events"


const string NW_TOTAL_MAP_PINS = "NW_TOTAL_MAP_PINS";
const string NW_MAP_PIN_NTRY_  = "NW_MAP_PIN_NTRY_";
const string NW_MAP_PIN_XPOS_  = "NW_MAP_PIN_XPOS_";
const string NW_MAP_PIN_YPOS_  = "NW_MAP_PIN_YPOS_";
const string NW_MAP_PIN_AREA_  = "NW_MAP_PIN_AREA_";


// -----------------------------------------------------------------------------
void LoadMapPins(object oPC);
void LoadMapPins(object oPC)
{
  if (!GetIsPC(oPC))
    return;

  // LogDebug("Map Pin: Loading map pins...");

  int nCount = NWNX_Object_GetInt(oPC, NW_TOTAL_MAP_PINS);
  SetLocalInt(oPC, NW_TOTAL_MAP_PINS, nCount);

  int i;
  for (i = 1; i <= nCount; i++)
  {
    string szi = IntToString(i);
    string sKey = NW_MAP_PIN_AREA_ + szi;

    string sTag = NWNX_Object_GetString(oPC, sKey + "_TAG");

    object oArea = GetObjectByTag(sTag);
    SetLocalObject(oPC, sKey, oArea);

    sKey = NW_MAP_PIN_NTRY_ + szi; SetLocalString(oPC, sKey, NWNX_Object_GetString(oPC, sKey));
    sKey = NW_MAP_PIN_XPOS_ + szi; SetLocalFloat(oPC,  sKey, NWNX_Object_GetFloat(oPC,  sKey));
    sKey = NW_MAP_PIN_YPOS_ + szi; SetLocalFloat(oPC,  sKey, NWNX_Object_GetFloat(oPC,  sKey));

    // LogDebug("Map Pin: Loaded pin " + szi + " for area " + GetObjectInfo(oArea));
  }

  // LogDebug("Map Pin: Loaded map pins.");
}


// -----------------------------------------------------------------------------
void SaveMapPins(object oPC);
void SaveMapPins(object oPC)
{
  if (!GetIsPC(oPC))
    return;

  LogDebug("Map Pin: Saving map pins...");

  // When saving, all pins should be valid area
  int i = 1;
  object oArea = GetLocalObject(oPC, NW_MAP_PIN_AREA_ + "1");
  while (GetIsObjectValid(oArea))
  {
    string szi = IntToString(i);
    string sKey = NW_MAP_PIN_AREA_ + szi;

    NWNX_Object_SetString(oPC, sKey + "_TAG", GetTag(oArea), TRUE);

    sKey = NW_MAP_PIN_NTRY_ + szi; NWNX_Object_SetString(oPC, sKey, GetLocalString(oPC, sKey), TRUE);
    sKey = NW_MAP_PIN_XPOS_ + szi; NWNX_Object_SetFloat(oPC,  sKey, GetLocalFloat(oPC, sKey),  TRUE);
    sKey = NW_MAP_PIN_YPOS_ + szi; NWNX_Object_SetFloat(oPC,  sKey, GetLocalFloat(oPC, sKey),  TRUE);

    LogDebug("Map Pin: Saved pin " + szi + " for " + GetObjectInfo(oArea));

    i += 1;
    szi = IntToString(i);
    oArea = GetLocalObject(oPC, NW_MAP_PIN_AREA_ + szi);
  }

  int nCount = i - 1;
  SetLocalInt(oPC, NW_TOTAL_MAP_PINS, nCount);
  NWNX_Object_SetInt(oPC, NW_TOTAL_MAP_PINS, nCount, TRUE);

  LogDebug("Map Pin: Saved map pins.");
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      // Areas can be recreated, thus invalidating the map pins area.
      // RegisterEvent(EVENT_CLIENT_ENTER);
      // RegisterEvent(EVENT_AREA_ENTER);
      // RegisterEvent(EVENT_AREA_DESTROY_BEFORE);
      // RegisterEvent(EVENT_AREA_CREATED);

      // RegisterEvent(EVENT_MAP_PIN_ADD_AFTER);
      // RegisterEvent(EVENT_MAP_PIN_CHANGE_AFTER);
      // RegisterEvent(EVENT_MAP_PIN_DESTROY_AFTER);
      return;

    case EVENT_CLIENT_ENTER:
    case EVENT_AREA_ENTER:
      LoadMapPins(GetEnteringObject());
      return;

    case EVENT_PLAYER_REST_STARTED:
      LoadMapPins(GetLastPCRested());
      return;

    case EVENT_AREA_CREATED:
      LoadMapPins(OBJECT_SELF);
      return;

    case EVENT_AREA_DESTROY_BEFORE:
    case EVENT_MAP_PIN_ADD_AFTER:
    case EVENT_MAP_PIN_CHANGE_AFTER:
    case EVENT_MAP_PIN_DESTROY_AFTER:
      SaveMapPins(OBJECT_SELF);
      return;
  }
}

