// =============================================================================
// Framework Unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Prevent duplicates in a story inventory.

Prevent duplicates in a story inventory by deleting a copy sold to the store if
it also exists as an infinite object.

Script does not check if item has same appearance.
*/

#include "fi_events"
#include "fi_item"

#include "nwnx_store"

// -----------------------------------------------------------------------------
void OnStoreAfterBuy();
void OnStoreAfterBuy()
{
  object oStore = StringToObject(NWNX_Events_GetEventData("STORE"));
  object oNewItem = StringToObject(NWNX_Events_GetEventData("ITEM"));
  int nPrice = StringToInt(NWNX_Events_GetEventData("PRICE"));

  // nResult is FALSE if transaction didn't happen
  int nResult = StringToInt(NWNX_Events_GetEventData("RESULT"));
  if (!nResult)
    return;

  object oItem = GetFirstItemInInventory(oStore);
  while (GetIsObjectValid(oItem))
  {
    if (GetTag(oItem) == GetTag(oNewItem) && GetResRef(oItem) == GetResRef(oNewItem))
    {
      if (oNewItem == oItem)
      {
        oItem = GetNextItemInInventory(oStore);
        continue;
      }

      if (GetInfiniteFlag(oItem) == FALSE)
      {
        oItem = GetNextItemInInventory(oStore);
        continue;
      }

      LogInfo("DestroyObject: " + IntToString(nPrice));
      DestroyObject(oNewItem, 0.0f);
    }
    oItem = GetNextItemInInventory(oStore);
  }
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();
  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_STORE_SELL_AFTER);
      return;

    case EVENT_STORE_SELL_AFTER:
      OnStoreAfterBuy();
      return;
  }
}

