// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Close the door after timeout.

Variables
~~~~~~~~~

:ev:`AUTOCLOSE`: *(int)* Close the door after 12 seconds.

Scripts
~~~~~~~

:ev:`OnOpen`: :file:`fw_event`
*/


#include "fi_events"
#include "fi_console"

#include "nwnx_object"

const string AUTOCLOSE = "AUTOCLOSE";


// =============================================================================
void CloseDoor(object oDoor);
void CloseDoor(object oDoor)
{
  ActionCloseDoor(oDoor);
}

void OnOpen(object oDoor);
void OnOpen(object oDoor)
{
  int nValue = GetLocalInt(oDoor, AUTOCLOSE);
  if (!nValue)
    return;

  float fValue = 12.0f;
  DelayCommand(fValue, CloseDoor(oDoor));
}

// =============================================================================
void main()
{
  object oArea = OBJECT_SELF;

  int nEvent = GetEvent();
  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_OBJECT_OPENED);
      return;

    case EVENT_OBJECT_OPENED:
      if (!NWNX_Object_GetDoorHasVisibleModel(OBJECT_SELF))
        return;
      OnOpen(OBJECT_SELF);
      return;
  }
}
