// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Set name of all object to the value of :ev:`NAME`.

The name will be set on the English localized string only, thus not interfering
with optional internationalization efforts.

Variables
~~~~~~~~~

:ev:`NAME`: *(string/int)* Name of the placeable, creature or area.
*/

#include "fi_area"
#include "fi_core"
#include "fi_events"
#include "fi_string"

#include "nwnx_object"


const string NAME = "NAME";


// -----------------------------------------------------------------------------
// If NAME var is set on the object, use it, otherwise, strip the ()
string GetNameEx(object oObject);
string GetNameEx(object oObject)
{
  string sName = GetLocalString(oObject, NAME);
  if (sName != "")
    return sName;

  sName = GetName(oObject, TRUE);
  return GetName(oObject);
}


// -----------------------------------------------------------------------------
// If NAME is set on the object, use it,
// Otherwise, strip the ()
void SetNameEx(object oObject);
void SetNameEx(object oObject)
{
  if (!GetIsObjectValid(oObject))
    return;

  string sName    = GetName(oObject, TRUE);
  string sNewName = GetNameEx(oObject);

  // Don't overwrite to avoid removing translations.
  if (sNewName != sName)
  {
    SetName(oObject, "");
    NWNX_Object_SetLocalizedName(oObject, sNewName, PLAYER_LANGUAGE_ENGLISH);
  }
}


// -----------------------------------------------------------------------------
void DoEventLoad();
void DoEventLoad()
{
  // Do all area names
  object oArea = GetFirstArea();
  while (GetIsObjectValid(oArea))
  {
    // SetNameEx(oArea);
    oArea = GetNextArea();
  }

  // Do all objects in this area only
  object oObject = GetFirstObjectInArea(OBJECT_SELF);
  while (GetIsObjectValid(oObject))
  {
    SetNameEx(oObject);
    oObject = GetNextObjectInArea();
  }
}


// =============================================================================
void main()
{
  object oPC = GetEnteringObject();
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_AREA_ENTER_ONCE);
      RegisterEvent(EVENT_AREA_CREATED);
      RegisterEvent(EVENT_NPC_SPAWN);
      return;

    case EVENT_AREA_ENTER_ONCE:
    case EVENT_AREA_CREATED:
      DoEventLoad();
      return;

    case EVENT_NPC_SPAWN:
      SetNameEx(OBJECT_SELF);
      return;
  }
}

