// =============================================================================
// Framework fix: Fix Trackless Step for underground natural areas
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Log generic events.
*/

// TODO: Log NWNX_ON_PARTY_*_BEFORE LEAVE, KICK, ACCEPT_INVITATION


#include "fi_events"

#include "fi_pc"

#include "nwnx_area"
#include "nwnx_player"


// -----------------------------------------------------------------------------
void OnClientEnter();
void OnClientEnter()
{
  object oWho = GetEnteringObject();

  LogNotice(
      "Client '" + GetPCPlayerName(oWho) + "' entering the module with " + GetObjectInfo(oWho)
    + ", IP = " + GetPCIPAddress(oWho)
    + ", level = " + IntToString(GetHitDice(oWho))
    + ", cdkey = " + GetPCPublicCDKey(oWho)
    + ", bicfile = " + NWNX_Player_GetBicFileName(oWho),
    CONSOLE_COLOR_GREEN
  );
}


// -----------------------------------------------------------------------------
void OnClientLeave();
void OnClientLeave()
{
  object oWho = GetExitingObject();

  LogNotice(
      "Client '" + GetPCPlayerName(oWho) + "' exiting the module is " + GetObjectInfo(oWho)
    + ", IP = " + GetPCIPAddress(oWho)
    + ", level = " + IntToString(GetHitDice(oWho)),
    CONSOLE_COLOR_GREEN
  );
}


// =============================================================================
void main()
{
  object oPC;
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      RegisterEvent(EVENT_CLIENT_ENTER);
      RegisterEvent(EVENT_CLIENT_LEAVE);
      RegisterEvent(EVENT_AREA_ENTER);
      RegisterEvent(EVENT_AREA_EXIT);
      RegisterEvent(EVENT_PLAYER_SAVE_AFTER);
      RegisterEvent(EVENT_PLAYER_RAISED);
      RegisterEvent(EVENT_PLAYER_RESSURECTED);
      RegisterEvent(EVENT_PLAYER_DEATH);
      RegisterEvent(EVENT_PLAYER_DYING);
      return;

    case EVENT_CLIENT_ENTER: OnClientEnter(); return;
    case EVENT_CLIENT_LEAVE: OnClientLeave(); return;

    case EVENT_AREA_ENTER:
      oPC = GetEnteringObject();
      if (!GetIsPC(oPC))
        return;
      LogNotice("PC " + GetObjectInfo(oPC) + " enters area " + GetObjectInfo(OBJECT_SELF));
      return;

    case EVENT_AREA_EXIT:
      oPC = GetExitingObject();
      if (!GetIsPC(oPC))
        return;
      LogNotice("PC " + GetObjectInfo(oPC) + " exit area " + GetObjectInfo(OBJECT_SELF));
      return;

    case EVENT_PLAYER_SAVE_AFTER:
      LogNotice(GetObjectInfo(OBJECT_SELF) + " was exported to servervault.");
      return;

    case EVENT_PLAYER_RAISED:
      oPC = OBJECT_SELF;
      LogNotice(GetObjectInfo(oPC) + " was raised in " + GetObjectInfo(GetArea(oPC))
        + IntToString(NWNX_Area_GetNumberOfPlayersInArea(GetArea(oPC))) + " players in area");
      return;

    case EVENT_PLAYER_RESSURECTED:
      oPC = OBJECT_SELF;
      LogNotice(GetObjectInfo(oPC) + " was ressurected in " + GetObjectInfo(GetArea(oPC))
        + IntToString(NWNX_Area_GetNumberOfPlayersInArea(GetArea(oPC))) + " players in area");
      return;

    case EVENT_PLAYER_DEATH:
      oPC = GetLastPlayerDied();
      LogNotice(GetObjectInfo(oPC) + " has died in " + GetObjectInfo(GetArea(oPC))
        + IntToString(NWNX_Area_GetNumberOfPlayersInArea(GetArea(oPC))) + " players in area.");
      return;
  }
}
