// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Log reloading events.
*/

#include "fi_area"
#include "fi_console"
#include "fi_events"

#include "nwnx_player"


// -----------------------------------------------------------------------------
void OnResourceChange(string sAction);
void OnResourceChange(string sAction)
{
  /*
    Event Data Tag        | Type   | Notes
    ----------------------|--------|-------
    ALIAS                 | string | NWNX for /nwnx, DEVELOPMENT for /development. Also supports valid aliases from the Custom Resman Definition File
    RESREF                | string | The ResRef of the file
    TYPE                  | int    | The type of the file, see NWNX_UTIL_RESREF_TYPE_*
  */
  string sAlias  = NWNX_Events_GetEventData("ALIAS");
  string sResRef = NWNX_Events_GetEventData(RESREF);
  int nType      = StringToInt(NWNX_Events_GetEventData("TYPE"));

  if (nType == 2009) // NWNX_UTIL_RESREF_TYPE_NSS
    return;

  string sType = "";
  switch (nType)
  {
    case NWNX_UTIL_RESREF_TYPE_AREA_ARE:  sType = "area"; break;
    case NWNX_UTIL_RESREF_TYPE_AREA_GIT:  sType = "area"; RecreateArea(sResRef); break;
    case NWNX_UTIL_RESREF_TYPE_CREATURE:  sType = "creature"; break;
    case NWNX_UTIL_RESREF_TYPE_DIALOG:    sType = "dialog"; break;
    case NWNX_UTIL_RESREF_TYPE_DOOR:      sType = "door"; break;
    case NWNX_UTIL_RESREF_TYPE_ENCOUNTER: sType = "encounter"; break;
    case NWNX_UTIL_RESREF_TYPE_ITEM:      sType = "item"; break;
    case NWNX_UTIL_RESREF_TYPE_NCS:       sType = "script"; break;
    case NWNX_UTIL_RESREF_TYPE_NSS:       return;
    case NWNX_UTIL_RESREF_TYPE_PLACEABLE: sType = "placeable"; break;
    case NWNX_UTIL_RESREF_TYPE_SOUND:     sType = "sound"; break;
    case NWNX_UTIL_RESREF_TYPE_STORE:     sType = "store"; break;
    case NWNX_UTIL_RESREF_TYPE_TRIGGER:   sType = "trigger"; break;
    case NWNX_UTIL_RESREF_TYPE_TWODA:     sType = "2da"; break;
    case NWNX_UTIL_RESREF_TYPE_WAYPOINT:  sType = "waypoint"; break;
    case 2014: sType = "module";  break;
    case 2030: sType = "palette"; break; // eg: 'itempalcus'
    case 2046: sType = "area";    break; // eg: area_3338.git
    default:
      sType = IntToString(nType);
      break;
  }

  string sLog = "Resource '" + sResRef + "' (" + sType + ") was " + sAction + " " + sAlias + " folder.";

  LogNotice(sLog, CONSOLE_COLOR_YELLOW);
}


// =============================================================================
void main()
{
  object oPC;
  int nEvent = GetEvent();

  switch (nEvent)
  {
    case EVENT_REGISTER:
      LogInfo("Registered reloader for resources.");
      RegisterEvent(EVENT_RESOURCE_ADDED);
      RegisterEvent(EVENT_RESOURCE_REMOVED);
      RegisterEvent(EVENT_RESOURCE_MODIFIED);
      return;

    case EVENT_RESOURCE_ADDED:    OnResourceChange("added to");     return;
    case EVENT_RESOURCE_REMOVED:  OnResourceChange("removed from"); return;
    case EVENT_RESOURCE_MODIFIED: OnResourceChange("modified in");  return;
  }
}
