// =============================================================================
// Framework unit
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Default respawn scripts.

To override with your personal respawn system, you must override the default scripts with
fw_event.

One way is to override the scripts in EVENT_REGISTER in your user unit:

.. code-block:: c

    case EVENT_REGISTER:
      SetEventScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_DEATH, SCRIPT_FW_EVENT);
      SetEventScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_DYING, SCRIPT_FW_EVENT);
      SetEventScript(oModule, EVENT_SCRIPT_MODULE_ON_RESPAWN_BUTTON_PRESSED, SCRIPT_FW_EVENT);

Another way is to remove this file, and set :file:`fw_event` as the script for all three events: OnPlayerDeath, OnPlayerDying, OnPlayerRespawn.
*/

// TODO: These scripts should Broadcast events

#include "fi_events"


// =============================================================================
void main()
{
  object oModule = GetModule();

  int nEvent = GetEvent();
  switch (nEvent)
  {
    case EVENT_REGISTER:
      SetEventScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_DEATH, "nw_o0_death");
      SetEventScript(oModule, EVENT_SCRIPT_MODULE_ON_PLAYER_DYING, "nw_o0_dying");
      SetEventScript(oModule, EVENT_SCRIPT_MODULE_ON_RESPAWN_BUTTON_PRESSED, "nw_o0_respawn");
      break;
  }
}
