// =============================================================================
// Framework dialog action
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Identify every object in inventory for the price of parameter :ev:`PRICE`.

Parameters
~~~~~~~~~~

:ev:`PRICE`: *(int)* Cost of the identification, per object.
*/

#include "fi_action"
#include "fi_const"
#include "fi_console"
#include "fi_message"


// =============================================================================
void main()
{
  object oPC    = GetPCSpeaker();
  int    nPrice = StringToInt(GetScriptParam(PRICE));
  int    nOnce  = TRUE;

  int nGold = GetGold(oPC);
  if (nGold < nPrice)
    return;

  object oObject = GetFirstItemInInventory(oPC);
  while (GetIsObjectValid(oObject))
  {
    if (GetIdentified(oObject))
    {
      oObject = GetNextItemInInventory(oPC);
      continue;
    }

    // Remove gold
    nGold = GetGold(oPC);
    if (nGold < nPrice)
    {
      SendFailureMessage(oPC, "Ran out of gold pieces");
      oObject = OBJECT_INVALID;
      continue;
    }

    if (nOnce)
      ActionCastFakeSpellAtObject(SPELL_IDENTIFY, GetLastSpeaker());
    nOnce = FALSE;

    TakeGoldFromCreature(nPrice, oPC, TRUE);

    SetIdentified(oObject, TRUE);
    SendSuccessMessage(oPC, "Identified a " + GetName(oObject));

    oObject = GetNextItemInInventory(oPC);
  }
}
