// =============================================================================
// Framework dialog action
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//! Open Store a store with appraisal only on charisma.
//!
//! TODO: Use a parameter to select if appraisal, no appraisal, or charisma is
//! to be used. Use APPRAISE_METHOD (Skill, Ability, None), MODIFIER (-5)
//!
//! Variables
//! ~~~~~~~~~
//!
//! :ev:`STORE`: *(string)* Tag of store to open


#include "fi_console"

#include "nwnx_store"


// (string)
// Placed as conversation param, or as a variable on NPC
// Store tag to be opened from dialog.
const string STORE = "STORE";


int GetBonusMarkUp(object oPC, object oStore);
int GetBonusMarkUp(object oPC, object oStore)
{
  int nModifier = GetAbilityModifier(ABILITY_CHARISMA, oPC);

  // Maximum modifier is 12, thus enabling up to 60% more or less markup.
  nModifier *= -5;

  return nModifier;
}


// =============================================================================
void main()
{
  object oPC = GetPCSpeaker();
  object oStore = OBJECT_INVALID;

  // Open store from dialog param
  string sStore = GetScriptParam(STORE);
  oStore = GetObjectByTag(sStore);

  if (!GetIsObjectValid(oStore) || GetObjectType(oStore) != OBJECT_TYPE_STORE)
  {
    // Open store from variable on creature which is in dialog
    sStore = GetLocalString(OBJECT_SELF, STORE);
    oStore = GetObjectByTag(sStore);
  }

  if (GetObjectType(oStore) == OBJECT_TYPE_STORE)
  {
    int nMarkUp   = NWNX_Store_GetMarkUp(oStore);
    int nMarkDown = NWNX_Store_GetMarkDown(oStore);

    int nBonusMarkUp = GetBonusMarkUp(oPC, oStore);
    int nBonusMarkDown = 0;

    if ((nMarkUp + nBonusMarkUp) > (nMarkDown + nBonusMarkDown))
    {
      // TODO: Do something to avoid this.

    }
    OpenStore(oStore, oPC, nBonusMarkUp);
    return;
  }

  string sLog = "could not find store " + (sStore != "" ? ("'" + sStore + "'") : "<unamed>");
  LogError(sLog + " from " + GetObjectInfo(OBJECT_SELF));
}
