// =============================================================================
// Framework dialog conditional
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Return TRUE or FALSE depending on the quantity of gold PC has.

Parameters
~~~~~~~~~~

:ev:`OP`: operator ``=``, ``>``, ``<``, ``<=``, ``>=``, ``!=``

:ev:`AMOUNT`: amount of gold to be checked
*/

#include "fi_const"
#include "fi_console"


// =============================================================================
int StartingConditional()
{
  object oSpeaker = GetPCSpeaker();
  string sOp      = GetScriptParam(OP);
  int    nAmount  = StringToInt(GetScriptParam(AMOUNT));

  int nGold = GetGold(oSpeaker);

  if (sOp == "=")
    return (nGold == nAmount);
  if (sOp == ">")
    return (nGold > nAmount);
  if (sOp == ">=")
    return (nGold >= nAmount);
  if (sOp == "<=")
    return (nGold <= nAmount);
  if (sOp == "<")
    return (nGold < nAmount);
  if (sOp == "!=")
    return (nGold != nAmount);

  // By default, you must have at least this amount of gold
  if (sOp == "")
    return (nGold >= nAmount);

  LogError("fw_dlg_cnd_gold: called with invalid 'OP' value");
  return FALSE;
}
