// =============================================================================
// Framework dialog conditional
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

/*! rst
Return TRUE if :ev:`AREA` and :ev:`VALUE` are correct.

The value of parameter :ev:`VALUE` will be checked against the local int :ev:`VALUE` stored on the PC.

Parameters
~~~~~~~~~~

:ev:`VALUE`: *(int)*

:ev:`AREA`:  *(string)*

Examples
~~~~~~~~

This script enables using a single dialog to show a different one liner depending on the PC's location (area) and a random value that was picked by :ref:`fw_dlg_cnd_rand`.

.. image:: /images/dialogs/fw_dlg_cnd_if001.png

*/

#include "fi_core"
#include "fi_console"


const string VALUE = "VALUE";


// =============================================================================
int StartingConditional()
{
  object oPC = GetLastSpeaker();
  string sArea = GetScriptParam("AREA");
  int nValue = StringToInt(GetScriptParam(VALUE));

  if (nValue != GetLocalInt(oPC, VALUE))
    return FALSE;

  if (sArea != GetTag(GetArea(oPC)))
    return FALSE;

  return TRUE;
}
