// =============================================================================
// Framework dialog conditional
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
//! Get the value of a variable by de-refencing it.
//!
//! Variables
//! ~~~~~~~~~
//!
//! :ev:`VARIABLE`: *(string)* Name of the variable
//!
//! :ev:`VALUE`: *(int)*  Value to check for.
//!
//! :ev:`OP`: operator ``=``, ``>``, ``<``, ``<=``, ``>=``, ``!=``
//!


#include "fi_console"


const string VARIABLE = "VARIABLE";
const string VALUE = "VALUE";


// =============================================================================
int StartingConditional()
{
  object oSpeaker  = OBJECT_SELF; // And not GetLastSpeaker() because of SpeakOneLinerConversation
  string sVariable = GetScriptParam(VARIABLE);
  int    nValue    = StringToInt(GetScriptParam(VALUE));
  string sOp       = GetScriptParam(OP);

  if (nValue == 0)
  {
    LogError("fw_dlg_cnd_var: called with invalid 'VALUE' value of 0");
    return FALSE;
  }

  int nVariable = GetLocalInt(oSpeaker, sVariable);
  if (nVariable == nValue)
    return TRUE;

  // By default, check equality
  if (sOp == "")
    return (nVariable == nValue);

  if (sOp == "=")
    return (nVariable == nValue);
  if (sOp == ">")
    return (nVariable > nValue);
  if (sOp == ">=")
    return (nVariable >= nValue);
  if (sOp == "<=")
    return (nVariable <= nValue);
  if (sOp == "<")
    return (nVariable < nValue);
  if (sOp == "!=")
    return (nVariable != nValue);

  LogError("fw_dlg_cnd_var: called with invalid 'OP' value");
  return FALSE;
}
// TODO: LogBug function, with details, of OBJECT_SELF, his location, etc.
