// TODO: Review, Test, Document
// =============================================================================
// Framework event script
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//

#include "fi_console"
#include "fi_string"
#include "fi_userdef"


// -----------------------------------------------------------------------------
// OnAreaEnter callsOnAreaLoad once per area after module is loaded.
void OnClientEnterOnce(object oPC);
void OnClientEnterOnce(object oPC)
{
  object oModule = GetModule();
  string sKey = "_EVENT_CLIENT_ENTER_ONCE_" + GetObjectUUID(oPC);

  if (GetLocalInt(oModule, sKey))
    return;
  SetLocalInt(oModule, sKey, TRUE);

  ExecuteUserDefinedScript(EVENT_CLIENT_ENTER_ONCE, oModule);
  BroadcastEvent(EVENT_CLIENT_ENTER_ONCE, oModule);
}

// -----------------------------------------------------------------------------
// OnAreaEnter callsOnAreaLoad once per area after module is loaded.
void OnAreaEnterOnce(object oArea);
void OnAreaEnterOnce(object oArea)
{
  if (GetLocalInt(oArea, "_EVENT_AREA_FIRST_ENTER_ONCE"))
    return;
  SetLocalInt(oArea, "_EVENT_AREA_FIRST_ENTER_ONCE", TRUE);

  ExecuteUserDefinedScript(EVENT_AREA_ENTER_ONCE);
  BroadcastEvent(EVENT_AREA_ENTER_ONCE);
}


// =============================================================================
// Main is wrapped in this function to add the EVENT_NWN_EVENT_* around this
// function
void main()
{
  object oModule = GetModule();
  object oWho = OBJECT_INVALID;
  object oItem = OBJECT_INVALID;

  int nEvent = GetCurrentlyRunningEvent(FALSE);
  if (nEvent == 0)
    nEvent = GetCurrentlyRunningEvent(TRUE);

  // LogDebug("Event: " + GetEventScriptName(nEvent) + " (" + IntToString(nEvent) + ") for " + GetObjectInfo(OBJECT_SELF));

  switch (nEvent)
  {
    // =========================================================================
    // MODULE
    // =========================================================================
    case EVENT_SCRIPT_MODULE_ON_MODULE_START: return;
    case EVENT_SCRIPT_MODULE_ON_MODULE_LOAD:            ExecuteUserDefinedScript(EVENT_MODULE_LOAD);    BroadcastEvent(EVENT_MODULE_LOAD);    return;
    case EVENT_SCRIPT_MODULE_ON_RESPAWN_BUTTON_PRESSED: ExecuteUserDefinedScript(EVENT_PLAYER_RESPAWN); BroadcastEvent(EVENT_PLAYER_RESPAWN); return;
    case EVENT_SCRIPT_MODULE_ON_NUI_EVENT:              ExecuteUserDefinedScript(EVENT_NUI);            BroadcastEvent(EVENT_NUI); return;

    // =========================================================================
    // ITEMS
    // =========================================================================
    // NOTE: This uses ExecuteItemScript and EVENT_ITEM's and not ExecuteUserDefinedScript
    // eg: For "itm_torch", itm_torch.nss will be first called, then BroadcastEvent to those who registered.
    case EVENT_SCRIPT_MODULE_ON_ACQUIRE_ITEM:  oItem = GetModuleItemAcquired();   ExecuteItemScript(oItem, EVENT_ITEM_ACQUIRE);   BroadcastEvent(EVENT_ITEM_ACQUIRE, oItem);   return;
    case EVENT_SCRIPT_MODULE_ON_LOSE_ITEM:     oItem = GetModuleItemLost();       ExecuteItemScript(oItem, EVENT_ITEM_UNACQUIRE); BroadcastEvent(EVENT_ITEM_UNACQUIRE, oItem); return;
    case EVENT_SCRIPT_MODULE_ON_ACTIVATE_ITEM: oItem = GetItemActivated();        ExecuteItemScript(oItem, EVENT_ITEM_ACTIVATE);  BroadcastEvent(EVENT_ITEM_ACTIVATE, oItem);  return;
    case EVENT_SCRIPT_MODULE_ON_EQUIP_ITEM:    oItem = GetPCItemLastEquipped();   ExecuteItemScript(oItem, EVENT_ITEM_EQUIP);     BroadcastEvent(EVENT_ITEM_EQUIP, oItem);     return;
    case EVENT_SCRIPT_MODULE_ON_UNEQUIP_ITEM:  oItem = GetPCItemLastUnequipped(); ExecuteItemScript(oItem, EVENT_ITEM_UNEQUIP);   BroadcastEvent(EVENT_ITEM_UNEQUIP, oItem);   return;


    // =========================================================================
    // STORES
    // =========================================================================
    // RegisterEvent only since there's no UserDefined store scripts
    case EVENT_SCRIPT_STORE_ON_OPEN:  BroadcastEvent(EVENT_STORE_OPEN);  return;
    case EVENT_SCRIPT_STORE_ON_CLOSE: BroadcastEvent(EVENT_STORE_CLOSE); return;


    // =========================================================================
    // TRIGGERS
    // =========================================================================
    // No need to RegisterEvent, only in Trigger UserDefined script
    case EVENT_SCRIPT_TRIGGER_ON_OBJECT_ENTER:  ExecuteUserDefinedScript(EVENT_TRIGGER_ENTER);     BroadcastEvent(EVENT_TRIGGER_ENTER);     return;
    case EVENT_SCRIPT_TRIGGER_ON_OBJECT_EXIT:   ExecuteUserDefinedScript(EVENT_TRIGGER_EXIT);      BroadcastEvent(EVENT_TRIGGER_EXIT);      return;
    case EVENT_SCRIPT_TRIGGER_ON_CLICKED:       ExecuteUserDefinedScript(EVENT_TRIGGER_CLICK);     BroadcastEvent(EVENT_TRIGGER_CLICK);     return;
    case EVENT_SCRIPT_TRIGGER_ON_HEARTBEAT:     ExecuteUserDefinedScript(EVENT_TRIGGER_HEARTBEAT); BroadcastEvent(EVENT_TRIGGER_HEARTBEAT); return;
    case EVENT_SCRIPT_TRIGGER_ON_TRAPTRIGGERED: ExecuteUserDefinedScript(EVENT_TRIGGER_TRIGGERED); BroadcastEvent(EVENT_TRIGGER_TRIGGERED); return;
    case EVENT_SCRIPT_TRIGGER_ON_DISARMED:      ExecuteUserDefinedScript(EVENT_TRIGGER_DISARM);    BroadcastEvent(EVENT_TRIGGER_DISARM);    return;


    // =========================================================================
    // ENCOUNTERS
    // =========================================================================
    case EVENT_SCRIPT_ENCOUNTER_ON_OBJECT_ENTER:        ExecuteUserDefinedScript(EVENT_ENCOUNTER_ENTER);     BroadcastEvent(EVENT_ENCOUNTER_ENTER);     return;
    case EVENT_SCRIPT_ENCOUNTER_ON_OBJECT_EXIT:         ExecuteUserDefinedScript(EVENT_ENCOUNTER_EXIT);      BroadcastEvent(EVENT_ENCOUNTER_EXIT);      return;
    case EVENT_SCRIPT_ENCOUNTER_ON_ENCOUNTER_EXHAUSTED: ExecuteUserDefinedScript(EVENT_ENCOUNTER_EXHAUSTED); BroadcastEvent(EVENT_ENCOUNTER_EXHAUSTED); return;

    // =========================================================================
    // CREATURES
    // =========================================================================
    // For every event we have to call the real handler, and that one will call the user-defined and broadcast
    // WARNING: fw_x_overrides temporary fixes
    case EVENT_SCRIPT_CREATURE_ON_END_COMBATROUND: ExecuteScript("x2_def_endcombat"); ExecuteUserDefinedScript(EVENT_NPC_END_ROUND);     BroadcastEvent(EVENT_NPC_END_ROUND);     return; // x2_def_endcombat, nw_c2_default3
    case EVENT_SCRIPT_CREATURE_ON_DIALOGUE:        ExecuteScript("x2_def_onconv");    ExecuteUserDefinedScript(EVENT_NPC_DIALOGUE);      BroadcastEvent(EVENT_NPC_DIALOGUE);      return; // x2_def_onconv,    nw_c2_default4
    case EVENT_SCRIPT_CREATURE_ON_MELEE_ATTACKED:  ExecuteScript("x2_def_attacked");  ExecuteUserDefinedScript(EVENT_NPC_ATTACKED);      BroadcastEvent(EVENT_NPC_ATTACKED);      return; // x2_def_attacked,  nw_c2_default5
    case EVENT_SCRIPT_CREATURE_ON_DAMAGED:         ExecuteScript("x2_def_ondamage");  ExecuteUserDefinedScript(EVENT_NPC_DAMAGED);       BroadcastEvent(EVENT_NPC_DAMAGED);       return; // x2_def_ondamage,  nw_c2_default6
    case EVENT_SCRIPT_CREATURE_ON_DEATH:           ExecuteScript("x2_def_ondeath");   ExecuteUserDefinedScript(EVENT_NPC_DEATH);         BroadcastEvent(EVENT_NPC_DEATH);         return; // x2_def_ondeath,   nw_c2_default7
    case EVENT_SCRIPT_CREATURE_ON_DISTURBED:       ExecuteScript("x2_def_ondisturb"); ExecuteUserDefinedScript(EVENT_NPC_DISTURBED);     BroadcastEvent(EVENT_NPC_DISTURBED);     return; // x2_def_ondisturb, nw_c2_default8
    case EVENT_SCRIPT_CREATURE_ON_SPAWN_IN:        ExecuteScript("x2_def_spawn");     ExecuteUserDefinedScript(EVENT_NPC_SPAWN);         BroadcastEvent(EVENT_NPC_SPAWN);         return; // x2_def_spawn,     nw_c2_default9
    case EVENT_SCRIPT_CREATURE_ON_RESTED:          ExecuteScript("x2_def_rested");    ExecuteUserDefinedScript(EVENT_NPC_RESTED);        BroadcastEvent(EVENT_NPC_RESTED);        return; // x2_def_rested,    nw_c2_defaulta
    case EVENT_SCRIPT_CREATURE_ON_SPELLCASTAT:     ExecuteScript("x2_def_spellcast"); ExecuteUserDefinedScript(EVENT_NPC_SPELL_CAST_AT); BroadcastEvent(EVENT_NPC_SPELL_CAST_AT); return; // x2_def_spellcast, nw_c2_defaultb
    case EVENT_SCRIPT_CREATURE_ON_BLOCKED_BY_DOOR: ExecuteScript("x2_def_onblocked"); ExecuteUserDefinedScript(EVENT_NPC_BLOCKED);       BroadcastEvent(EVENT_NPC_BLOCKED);       return; // x2_def_onblocked, nw_c2_defaulte

    // PC's can have scripts like creatures, so we have to differenciate
    // between NPC and PC.
    case EVENT_SCRIPT_CREATURE_ON_HEARTBEAT: // x2_def_heartbeat, nw_c2_default1
      oWho = OBJECT_SELF;
      if (GetIsPC(oWho))
        BroadcastEvent(EVENT_PLAYER_HEARTBEAT);
      else
        ExecuteScript("x2_def_heartbeat");
      ExecuteUserDefinedScript(EVENT_NPC_HEARTBEAT);
      BroadcastEvent(EVENT_NPC_HEARTBEAT);
      return;

    case EVENT_SCRIPT_CREATURE_ON_NOTICE: // x2_def_percept, nw_c2_default2
      oWho = OBJECT_SELF;
      if (GetIsPC(oWho))
        BroadcastEvent(EVENT_PLAYER_NOTICE);
      else
        ExecuteScript("x2_def_percept");
      ExecuteUserDefinedScript(EVENT_NPC_NOTICE);
      BroadcastEvent(EVENT_NPC_NOTICE);
      return;

    // =========================================================================
    // Client
    // =========================================================================
    case EVENT_SCRIPT_MODULE_ON_CLIENT_ENTER:
      // Set scripts on the PC
      oWho = GetEnteringObject();
      SetEventScript(oWho, EVENT_SCRIPT_CREATURE_ON_HEARTBEAT, SCRIPT_FW_EVENT);
      SetEventScript(oWho, EVENT_SCRIPT_CREATURE_ON_NOTICE, SCRIPT_FW_EVENT);

      // GetEntering / GetExiting requires OBJECT_SELF = oModule
      OnClientEnterOnce(oWho);
      ExecuteUserDefinedScript(EVENT_CLIENT_ENTER, oModule);
      BroadcastEvent(EVENT_CLIENT_ENTER, oModule);

      // The original module added delay was probably to avoid an exploit, so we'll
      // keep it. As for the rest of the module is delayed to after the load
      // script, keep one second as a minimum.
      AssignCommand(oWho, ActionWait(GetIsDM(oWho) ? 1.3f : 6.3f));
      return;

    case EVENT_SCRIPT_MODULE_ON_CLIENT_EXIT:
      // GetEntering / GetExiting requires OBJECT_SELF = oModule
      ExecuteUserDefinedScript(EVENT_CLIENT_LEAVE, oModule);
      BroadcastEvent(EVENT_CLIENT_LEAVE, oModule);
      return;


    // =========================================================================
    // AREAS
    // =========================================================================
    // RegisterEvent if you want to recieve all area enter/exit
    case EVENT_SCRIPT_AREA_ON_ENTER:
      OnAreaEnterOnce(OBJECT_SELF);

      ExecuteUserDefinedScript(EVENT_AREA_ENTER);
      BroadcastEvent(EVENT_AREA_ENTER);
      return;

    case EVENT_SCRIPT_AREA_ON_EXIT:
      ExecuteUserDefinedScript(EVENT_AREA_EXIT);
      BroadcastEvent(EVENT_AREA_EXIT);
      return;

    case EVENT_SCRIPT_AREA_ON_HEARTBEAT:
      // This can happen when using CreateArea/DestroyArea
      if (GetArea(OBJECT_SELF) == OBJECT_INVALID)
        return;
      ExecuteUserDefinedScript(EVENT_AREA_HEARTBEAT);
      return;


    // =========================================================================
    // PLACEABLES & DOORS
    // =========================================================================
    // In rare cases we differenciate.
    case EVENT_SCRIPT_PLACEABLE_ON_USED:
      ExecuteUserDefinedScript(EVENT_OBJECT_USED);
      BroadcastEvent(EVENT_OBJECT_USED);
      return;

    case EVENT_SCRIPT_DOOR_ON_UNLOCK:
    case EVENT_SCRIPT_PLACEABLE_ON_UNLOCK:
      ExecuteUserDefinedScript(EVENT_OBJECT_UNLOCKED);
      BroadcastEvent(EVENT_OBJECT_UNLOCKED);
      return;

    case EVENT_SCRIPT_DOOR_ON_CLOSE:
    case EVENT_SCRIPT_PLACEABLE_ON_CLOSED:
      ExecuteUserDefinedScript(EVENT_OBJECT_CLOSED);
      BroadcastEvent(EVENT_OBJECT_CLOSED);
      return;

    case EVENT_SCRIPT_DOOR_ON_LOCK:
    case EVENT_SCRIPT_PLACEABLE_ON_LOCK:
      ExecuteUserDefinedScript(EVENT_OBJECT_LOCKED);
      BroadcastEvent(EVENT_OBJECT_LOCKED);
      return;

    case EVENT_SCRIPT_DOOR_ON_OPEN:
    case EVENT_SCRIPT_PLACEABLE_ON_OPEN:
      ExecuteUserDefinedScript(EVENT_OBJECT_OPENED);
      BroadcastEvent(EVENT_OBJECT_OPENED);
      return;

    case EVENT_SCRIPT_DOOR_ON_DEATH:
    case EVENT_SCRIPT_PLACEABLE_ON_DEATH:
      ExecuteUserDefinedScript(EVENT_OBJECT_DEATH);
      BroadcastEvent(EVENT_OBJECT_DEATH);
      return;

    case EVENT_SCRIPT_DOOR_ON_DAMAGE:
    case EVENT_SCRIPT_PLACEABLE_ON_DAMAGED:
      ExecuteUserDefinedScript(EVENT_OBJECT_DAMAGED);
      BroadcastEvent(EVENT_OBJECT_DAMAGED);
      return;

    case EVENT_SCRIPT_DOOR_ON_SPELLCASTAT:
    case EVENT_SCRIPT_PLACEABLE_ON_SPELLCASTAT:
      ExecuteUserDefinedScript(EVENT_SPELL_CAST_AT);
      BroadcastEvent(EVENT_SPELL_CAST_AT);
      return;

    case EVENT_SCRIPT_DOOR_ON_HEARTBEAT:
    case EVENT_SCRIPT_PLACEABLE_ON_HEARTBEAT:
      ExecuteUserDefinedScript(EVENT_OBJECT_HEARTBEAT);
      BroadcastEvent(EVENT_OBJECT_HEARTBEAT);
      return;

    case EVENT_SCRIPT_PLACEABLE_ON_LEFT_CLICK:
      ExecuteUserDefinedScript(EVENT_OBJECT_CLICK);
      BroadcastEvent(EVENT_OBJECT_CLICK);
      return;

    case EVENT_SCRIPT_DOOR_ON_FAIL_TO_OPEN:
      ExecuteUserDefinedScript(EVENT_OBJECT_FAIL_TO_OPEN);
      BroadcastEvent(EVENT_OBJECT_FAIL_TO_OPEN);
      return;

    case EVENT_SCRIPT_DOOR_ON_MELEE_ATTACKED:
    case EVENT_SCRIPT_PLACEABLE_ON_MELEEATTACKED:
      ExecuteUserDefinedScript(EVENT_OBJECT_ATTACKED);
      BroadcastEvent(EVENT_OBJECT_ATTACKED);
      return;

    case EVENT_SCRIPT_PLACEABLE_ON_INVENTORYDISTURBED:
      switch (GetInventoryDisturbType())
      {
        case INVENTORY_DISTURB_TYPE_ADDED:   nEvent = EVENT_OBJECT_DISTURB_ADDED; break;
        case INVENTORY_DISTURB_TYPE_REMOVED: nEvent = EVENT_OBJECT_DISTURB_REMOVED; break;
        case INVENTORY_DISTURB_TYPE_STOLEN:  nEvent = EVENT_OBJECT_DISTURB_STOLEN; break;
      }
      ExecuteUserDefinedScript(nEvent);
      BroadcastEvent(nEvent);
      return;

    // =========================================================================
    // Player
    // =========================================================================
    case EVENT_SCRIPT_MODULE_ON_PLAYER_DEATH:           ExecuteUserDefinedScript(EVENT_PLAYER_DEATH); BroadcastEvent(EVENT_PLAYER_DEATH); return;
    case EVENT_SCRIPT_MODULE_ON_PLAYER_DYING:           ExecuteUserDefinedScript(EVENT_PLAYER_DYING); BroadcastEvent(EVENT_PLAYER_DYING); return;
    case EVENT_SCRIPT_MODULE_ON_PLAYER_LEVEL_UP:        ExecuteUserDefinedScript(EVENT_PLAYER_LEVELUP); BroadcastEvent(EVENT_PLAYER_LEVELUP, GetPCLevellingUp()); return;

    case EVENT_SCRIPT_MODULE_ON_PLAYER_TILE_ACTION:     ExecuteUserDefinedScript(EVENT_PLAYER_TILE_ACTION); BroadcastEvent(EVENT_PLAYER_TILE_ACTION); return;
    case EVENT_SCRIPT_MODULE_ON_PLAYER_TARGET:          ExecuteUserDefinedScript(EVENT_PLAYER_TARGET);       BroadcastEvent(EVENT_PLAYER_TARGET); return;

    case EVENT_SCRIPT_MODULE_ON_PLAYER_REST:
      oWho = GetLastPCRested();
      switch (GetLastRestEventType())
      {
        case REST_EVENTTYPE_REST_STARTED:   BroadcastEvent(EVENT_PLAYER_REST_STARTED, oWho);   return;
        case REST_EVENTTYPE_REST_CANCELLED: BroadcastEvent(EVENT_PLAYER_REST_CANCELLED, oWho); return;
        case REST_EVENTTYPE_REST_FINISHED:  BroadcastEvent(EVENT_PLAYER_REST_FINISHED, oWho); return;
      }
      return;

    case EVENT_SCRIPT_MODULE_ON_PLAYER_CHAT:
      {
        string sMessage = GetPCChatMessage();
        sMessage = GetStringTrim(GetStringLowerCase(sMessage));

        // Handle help
        if (FALSE
           || (sMessage == "/help") || (sMessage == "-help")
           || (sMessage == "/?")    || (sMessage == "-?")
           )
        {
          SetPCChatMessage("");
          BroadcastEvent(EVENT_PLAYER_CHAT_HELP);
          return;
        }

        BroadcastEvent(EVENT_PLAYER_CHAT_BEFORE);
        BroadcastEvent(EVENT_PLAYER_CHAT);
        BroadcastEvent(EVENT_PLAYER_CHAT_AFTER);
        return;
      }

    case EVENT_SCRIPT_MODULE_ON_PLAYER_GUIEVENT:
      switch (GetLastGuiEventType())
      {
        case GUIEVENT_EFFECTICON_CLICK:
          BroadcastEvent(EVENT_PLAYER_GUIEVENT_EFFECTICON_CLICK);
          return;
      }
      return;

    // =========================================================================
    // Unused
    // =========================================================================
    // Currently unused.
    /*
    case EVENT_SCRIPT_DOOR_ON_TRAPTRIGGERED:
    case EVENT_SCRIPT_PLACEABLE_ON_TRAPTRIGGERED:
    case EVENT_SCRIPT_TRIGGER_ON_TRAPTRIGGERED:
      ExecuteUserDefinedScript(EVENT_TRAP_TRIGGERED);
      BroadcastEvent(EVENT_TRAP_TRIGGERED);
      return;

    case EVENT_SCRIPT_DOOR_ON_DISARM:
    case EVENT_SCRIPT_PLACEABLE_ON_DISARM:
    case EVENT_SCRIPT_TRIGGER_ON_DISARMED:
      ExecuteUserDefinedScript(EVENT_OBJECT_TRAP_DISARMED);
      BroadcastEvent(EVENT_OBJECT_TRAP_DISARMED);
      return;

    case EVENT_SCRIPT_DOOR_ON_DIALOGUE:
    case EVENT_SCRIPT_PLACEABLE_ON_DIALOGUE:
    case EVENT_SCRIPT_DOOR_ON_CLICKED:
    case EVENT_SCRIPT_AREAOFEFFECT_ON_HEARTBEAT:
    case EVENT_SCRIPT_AREAOFEFFECT_ON_OBJECT_ENTER:
    case EVENT_SCRIPT_AREAOFEFFECT_ON_OBJECT_EXIT:
    case EVENT_SCRIPT_MODULE_ON_PLAYER_CANCEL_CUTSCENE:
    */
    default:
      LogInfo("Event `" + GetEventScriptName(nEvent) + "` (" + IntToString(nEvent) + ") not handled (fw_evt_nwn.nss)");
  }
}
