// =============================================================================
// Framework event script
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
// Sortable
//
// Register framework and user scripts.

#include "fi_console"
#include "fi_events"

#include "nwnx_events"

// -----------------------------------------------------------------------------

void _NWNX_Events_SubscribeEventEx(string sEvent);
void _NWNX_Events_SubscribeEventEx(string sEvent)
{
  if (NWNX_Events_GetNumSubscribers(sEvent))
    return;
  NWNX_Events_SubscribeEvent(sEvent, SCRIPT_FW_EVT_NWNX);
}


// -----------------------------------------------------------------------------

void _RefuseSubscription(string sEvent);
void _RefuseSubscription(string sEvent)
{
  LogWarning(sEvent + " should not be registered.");
}


// -----------------------------------------------------------------------------

void main()
{
  object oModule = GetModule();

  int nEventId = GetLocalInt(oModule, _REGISTER_EVENT_ID);
  string sUnit = GetLocalString(oModule, _REGISTER_EVENT_SCRIPT);
  string sEventName = GetEventName(nEventId);

  if (sEventName == "")
  {
    LogError("Tried to register event " + IntToString(nEventId)
      + " for unit '" + sUnit + "' that is unknown in the event database.");
    return;
  }

  // For any NWNX event, subscribe once
  switch (nEventId)
  {
    // Don't forget to update fw_evt_subscribe. fw_evt_nwnx, fi_events_const, fi_events_name
    case EVENT_CALENDAR_DAY:               _NWNX_Events_SubscribeEventEx(NWNX_ON_CALENDAR_DAY); break;
    case EVENT_CALENDAR_HOUR:              _NWNX_Events_SubscribeEventEx(NWNX_ON_CALENDAR_HOUR); break;
    case EVENT_CLIENT_CONNECT_AFTER:       _NWNX_Events_SubscribeEventEx(NWNX_ON_CLIENT_CONNECT_AFTER); break;
    case EVENT_CLIENT_DISCONNECT_BEFORE:   _NWNX_Events_SubscribeEventEx(NWNX_ON_CLIENT_DISCONNECT_BEFORE); break;
    case EVENT_ITEM_DROP_AFTER:            _NWNX_Events_SubscribeEventEx(NWNX_ON_INPUT_DROP_ITEM_AFTER); break;
    case EVENT_ITEM_DROP_BEFORE:           _NWNX_Events_SubscribeEventEx(NWNX_ON_INPUT_DROP_ITEM_BEFORE); break;
    case EVENT_MAP_PIN_ADD_AFTER:          _NWNX_Events_SubscribeEventEx(NWNX_ON_MAP_PIN_ADD_PIN_AFTER); break;
    case EVENT_MAP_PIN_CHANGE_AFTER:       _NWNX_Events_SubscribeEventEx(NWNX_ON_MAP_PIN_CHANGE_PIN_AFTER); break;
    case EVENT_MAP_PIN_DESTROY_AFTER:      _NWNX_Events_SubscribeEventEx(NWNX_ON_MAP_PIN_DESTROY_PIN_AFTER); break;
    case EVENT_PLAYER_ELC_VALIDATE_BEFORE: _NWNX_Events_SubscribeEventEx(NWNX_ON_ELC_VALIDATE_CHARACTER_BEFORE); break;
    case EVENT_PLAYER_LEVELDOWN_AFTER:     _NWNX_Events_SubscribeEventEx(NWNX_ON_LEVEL_DOWN_AFTER); break;
    case EVENT_PLAYER_LEVELUP_BEFORE:      _NWNX_Events_SubscribeEventEx(NWNX_ON_LEVEL_UP_BEFORE); break;
    case EVENT_PLAYER_SAVE_AFTER:          _NWNX_Events_SubscribeEventEx(NWNX_ON_SERVER_CHARACTER_SAVE_AFTER); break;
    case EVENT_REPUTATION_CHANGE_BEFORE:   _NWNX_Events_SubscribeEventEx(NWNX_ON_SET_NPC_FACTION_REPUTATION_BEFORE); break;
    case EVENT_RESOURCE_ADDED:             _NWNX_Events_SubscribeEventEx(NWNX_ON_RESOURCE_ADDED); break;
    case EVENT_RESOURCE_MODIFIED:          _NWNX_Events_SubscribeEventEx(NWNX_ON_RESOURCE_MODIFIED); break;
    case EVENT_RESOURCE_REMOVED:           _NWNX_Events_SubscribeEventEx(NWNX_ON_RESOURCE_REMOVED); break;
    case EVENT_STORE_SELL_AFTER:           _NWNX_Events_SubscribeEventEx(NWNX_ON_STORE_REQUEST_SELL_AFTER); break;
    case EVENT_STORE_SELL_BEFORE:          _NWNX_Events_SubscribeEventEx(NWNX_ON_STORE_REQUEST_SELL_BEFORE); break;
    case EVENT_FACTION_REPUTATION_BEFORE:  _NWNX_Events_SubscribeEventEx(NWNX_ON_SET_NPC_FACTION_REPUTATION_BEFORE); break;

    case EVENT_MODULE_START:     _RefuseSubscription(sEventName); return;
    case EVENT_OBJECT_TRACKED:   _RefuseSubscription(sEventName); return;
    case EVENT_OBJECT_UNTRACKED: _RefuseSubscription(sEventName); return;
    case EVENT_REGISTER:         _RefuseSubscription(sEventName); return;
  }

  Array_PushBack_Str(sEventName, sUnit);
  int nIndex = Array_Find_Str(sEventName, sUnit);

  LogInfo("  Registered unit '" + sUnit + "'"
    + " at index: " + IntToString(nIndex)
    + " for event " + GetEventNameEx(nEventId));
}
