// =============================================================================
// Framework fire and forget
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw

//! Sends an event every real time X (minute, hour, day), and once a minute with
//! a random offset.

#include "fi_userdef"


// -----------------------------------------------------------------------------

void _PulseFloatingMinute();
void _PulseFloatingMinute()
{
  object oModule = GetModule();
  ExecuteUserDefinedScript(EVENT_MODULE_FLOATING_MINUTE, oModule);
  BroadcastEvent(EVENT_MODULE_FLOATING_MINUTE, oModule);
}


// -----------------------------------------------------------------------------

void _PulseMinute();
void _PulseMinute()
{
  object oModule = GetModule();

  ExecuteUserDefinedScript(EVENT_MODULE_REALTIME_MINUTE, oModule);
  BroadcastEvent(EVENT_MODULE_REALTIME_MINUTE, oModule);

  DelayCommand(IntToFloat(Random(10)), _PulseFloatingMinute());
  DelayCommand(60.0f, _PulseMinute());
}


// -----------------------------------------------------------------------------

void _PulseHour();
void _PulseHour()
{
  object oModule = GetModule();
  ExecuteUserDefinedScript(EVENT_MODULE_REALTIME_HOUR, oModule);
  BroadcastEvent(EVENT_MODULE_REALTIME_HOUR, oModule);
  DelayCommand(3600.0f, _PulseHour());
}


// -----------------------------------------------------------------------------

void _PulseDay();
void _PulseDay()
{
  object oModule = GetModule();
  ExecuteUserDefinedScript(EVENT_MODULE_REALTIME_DAY, oModule);
  BroadcastEvent(EVENT_MODULE_REALTIME_DAY, oModule);
  DelayCommand(24 * 3600.0f, _PulseDay());
}


// =============================================================================
void main()
{
  // We have heartbeats for placeables and the such, but we can also tick
  // ourselves.
  _PulseMinute();
  _PulseHour();
  _PulseDay();
}
