//:://////////////////////////////////////////////////
//:: NW_C2_DEFAULT6
//:: Default OnDamaged handler
/*
    If already fighting then ignore, else determine
    combat round
 */
//:://////////////////////////////////////////////////
//:: Copyright (c) 2002 Floodgate Entertainment
//:: Created By: Naomi Novik
//:: Created On: 12/22/2002
//:://////////////////////////////////////////////////
//:://////////////////////////////////////////////////
//:: Modified By: Deva Winblood
//:: Modified On: Jan 17th, 2008
//:: Added Support for Mounted Combat Feat Support
//:://////////////////////////////////////////////////

#include "fi_events"
#include "fi_userdef"

#include "nw_i0_generic"

#include "nwnx_creature"

void OnDamage();
void OnDamage()
{
  object oDamager = GetLastDamager();
  if (!GetIsObjectValid(oDamager))
    return;

  int nFaction = NWNX_Creature_GetFaction(OBJECT_SELF);
  // TODO: Flee if AI_ON_ATTACK_FLEE or if Standard faction

  AI_DetermineCombatRound(oDamager);
  // GetIsFighting
  // GetObjectSeen

  if (GetSpawnInCondition(NW_FLAG_SET_WARNINGS))
    { /* nothing, should warn the attacker */ }
  else {
    // TODO: AI
    if (!GetIsFighting(OBJECT_SELF)) {
            // If we're not fighting, determine combat round
            if(GetBehaviorState(NW_FLAG_BEHAVIOR_SPECIAL)) {
                DetermineSpecialBehavior(oDamager);
            } else {
                if(!GetObjectSeen(oDamager)
                   && GetArea(OBJECT_SELF) == GetArea(oDamager)) {
                    // We don't see our attacker, go find them
                    ActionMoveToLocation(GetLocation(oDamager), TRUE);
                    ActionDoCommand(DetermineCombatRound());
                } else {
                    DetermineCombatRound();
                }
            }
        } else {
            // We are fighting already -- consider switching if we've been
            // attacked by a more powerful enemy
            object oTarget = GetAttackTarget();
            if (!GetIsObjectValid(oTarget))
                oTarget = GetAttemptedAttackTarget();
            if (!GetIsObjectValid(oTarget))
                oTarget = GetAttemptedSpellTarget();

            // If our target isn't valid
            // or our damager has just dealt us 25% or more
            //    of our hp in damager
            // or our damager is more than 2HD more powerful than our target
            // switch to attack the damager.
            if (!GetIsObjectValid(oTarget)
                || (
                    oTarget != oDamager
                    &&  (
                         GetTotalDamageDealt() > (GetMaxHitPoints(OBJECT_SELF) / 4)
                         || (GetHitDice(oDamager) - 2) > GetHitDice(oTarget)
                         )
                    )
                )
            {
                // Switch targets
                DetermineCombatRound(oDamager);
            }
        }
    }

  // Send the user-defined event signal
  if (GetSpawnInCondition(NW_FLAG_DAMAGED_EVENT))
    SignalEvent(OBJECT_SELF, EventUserDefined(EVENT_DAMAGED));
}

void main()
{
  // Plot objects ignore everything
  if (GetPlotFlag(OBJECT_SELF))
    return;

  OnDamage();

  ExecuteUserDefinedScript(EVENT_NPC_DAMAGED);
  BroadcastEvent(EVENT_NPC_DAMAGED);
}
