// =============================================================================
// Framework fix: Continual Light
// =============================================================================
// NWN Faerun Framework (c) 2024 by Julien Lecomte
// SPDX-License-Identifier: MIT
//
// This file is part of the Faerun Framework:
//   https://framework.faerun.pw
//
// cf. x0_s0_clight
//
// Continual light will position a variable on the item ("x0_s0_clight").
// Cast it again on the item every time it's re-acquired, notably after a server
// restart.
//
//! Prevent exploits by making property non-permanent


#include "fi_events"
#include "fi_console"

#include "x2_inc_spellhook"


// =============================================================================
void ContinualLightOn(object oItem, object oOwner);
void ContinualLightOn(object oItem, object oOwner)
{
  if (!GetIsObjectValid(oItem))
    return;

  // Only if item had continual light cast upon it.
  if (!NWNX_Object_GetInt(oItem, "x0_s0_clight"))
    return;

  float fDuration = HoursToSeconds(365 * 24 * 10);
  itemproperty ip = ItemPropertyLight(IP_CONST_LIGHTBRIGHTNESS_BRIGHT, IP_CONST_LIGHTCOLOR_WHITE);
  IPSafeAddItemProperty(oItem, ip, fDuration, X2_IP_ADDPROP_POLICY_REPLACE_EXISTING, TRUE, TRUE);
}


// =============================================================================
void main()
{
  int nEvent = GetEvent();
  switch (nEvent)
  {
    case EVENT_REGISTER:
      NWNX_Util_SetResourceOverride(RESTYPE_NCS, "x0_s0_clight", "fw_ovr_clight");
      RegisterEvent(EVENT_ITEM_ACQUIRE);
      break;

    case EVENT_ITEM_ACQUIRE:
      ContinualLightOn(OBJECT_SELF, GetModuleItemAcquiredBy());
      break;
  }
}
