// TODO: Review, Test, Document
// =============================================================================
// Framework fix: Mule
// =============================================================================
//
// Try to prevent some muling.
//
// Set a variable on the object when dropped. Remove it when acquired.
// If the same CDKEY tries to pick up the item, then there's some muling
// involved.


#include "fi_console"
#include "fi_events"
#include "fi_message"

#include "nwnx_player"


const string MULE_DROPPED_BY_CDKEY = "MULE_DROPPED_BY_CDKEY";
const string MULE_DROPPED_BY_BIC = "MULE_DROPPED_BY_BIC";


// -----------------------------------------------------------------------------
void OnUnacquire();
void OnUnacquire()
{
  object oPC = GetModuleItemLostBy();
  object oObject = GetModuleItemLost();

  if (!GetIsObjectValid(oObject))
    return;

  if (!GetIsObjectValid(oPC))
    return;

  if (!GetIsPC(oPC))
    return;

  if (GetIsPlayerDM(oPC))
    return;

  string sBIC = NWNX_Player_GetBicFileName(oPC);
  string sCDKEY = GetPCPublicCDKey(oPC);
  if (sCDKEY == "")
    return;

  SetLocalString(oObject, MULE_DROPPED_BY_CDKEY, sCDKEY);
  SetLocalString(oObject, MULE_DROPPED_BY_BIC,   sBIC);

  // DebugPrint("SetLocalString => " +  sCDKEY + ", " + sBIC);
}


// -----------------------------------------------------------------------------
void ValidateAcquire(object oPC, object oObject, object oFrom);
void ValidateAcquire(object oPC, object oObject, object oFrom)
{
  if (!GetIsObjectValid(oObject))
    return;

  if (!GetIsObjectValid(oPC))
    return;

  if (!GetIsPC(oPC))
    return;

  if (GetIsPlayerDM(oPC))
    return;

  // It has to be invalid (OBJECT_INVALID) because otherwise it's not muling
  if (GetIsObjectValid(oFrom))
    return;

  string sCDKEY1 = GetLocalString(oObject, MULE_DROPPED_BY_CDKEY);
  string sBIC1   = GetLocalString(oObject, MULE_DROPPED_BY_BIC);
  string sCDKEY2 = GetPCPublicCDKey(oPC);
  string sBIC2   = NWNX_Player_GetBicFileName(oPC);

  // Player to player donation.
  if (sCDKEY1 != sCDKEY2)
    return;

  // Self to self
  if (sBIC1 == sBIC2)
    return;
  else
  {
    LogWarning("exploit: Possible muling from " + GetObjectInfo(oPC));

    string sWarn = "WARNING! Possible muling detected. Muling is considered as exploiting. Muling will result in a ban.";
    // TODO: Setup a ban policy.
    SendFailureMessage(oPC, sWarn);
    FloatingTextStringOnCreature(sWarn, oPC, FALSE, FALSE);

  }
}

// -----------------------------------------------------------------------------
void OnAcquire();
void OnAcquire()
{
  object oObject = GetModuleItemAcquired();
  object oPC = GetModuleItemAcquiredBy();
  object oFrom = GetModuleItemAcquiredFrom();

  ValidateAcquire(oPC, oObject, oFrom);

  DeleteLocalString(oObject, MULE_DROPPED_BY_CDKEY);
  DeleteLocalString(oObject, MULE_DROPPED_BY_BIC);

  //DebugPrint("DeleteLocalString");
}

// =============================================================================
void main()
{
  int nEvent = GetEvent();
  switch (nEvent)
  {
    case EVENT_REGISTER:
      // TODO: Disabled, need more testing
      // RegisterEvent(EVENT_ITEM_ACQUIRE);
      // RegisterEvent(EVENT_ITEM_UNACQUIRE);
      break;

    case EVENT_ITEM_ACQUIRE:
      OnAcquire();
      break;

    case EVENT_ITEM_UNACQUIRE:
      OnUnacquire();
      break;
  }
}
