// =============================================================================
// Continual Light
// =============================================================================
// Copyright (c) 2002 Bioware Corp.
//
// Permanent Light spell
//
//   If cast on an item, item will get permanently
//   get the property "light".
//   Previously existing permanent light properties
//   will be removed!
//
// Fix:
//   Prevent exploits by making property non-permanent
//   but long duration (10 years)
//
// cf. fw_un_fix_clight, x0_s0_clight

#include "x2_inc_spellhook"

#include "nwnx_object"


// =============================================================================
void main()
{
  if (!X2PreSpellCastCode())
    return;

  int nDuration;
  int nMetaMagic;

  object oTarget = GetSpellTargetObject();

  // Handle spell cast on item....
  if (GetObjectType(oTarget) == OBJECT_TYPE_ITEM && ! CIGetIsCraftFeatBaseItem(oTarget))
  {
    // Do not allow casting on not equippable items
    if (!IPGetIsItemEquipable(oTarget))
    {
      // Item must be equipable...
      FloatingTextStrRefOnCreature(83326, OBJECT_SELF);
      return;
    }
    float fDuration = HoursToSeconds(365 * 24 * 10);
    itemproperty ip = ItemPropertyLight(IP_CONST_LIGHTBRIGHTNESS_BRIGHT, IP_CONST_LIGHTCOLOR_WHITE);
    IPSafeAddItemProperty(oTarget, ip, fDuration, X2_IP_ADDPROP_POLICY_REPLACE_EXISTING, TRUE, TRUE);
    NWNX_Object_SetInt(oTarget, "x0_s0_clight", TRUE, TRUE);
  }
  else
  {
    // Declare major variables
    effect eVis = (EffectVisualEffect(VFX_DUR_LIGHT_WHITE_20));
    effect eDur = EffectVisualEffect(VFX_DUR_CESSATE_POSITIVE);
    effect eLink = SupernaturalEffect(EffectLinkEffects(eVis, eDur));

    // Fire cast spell at event for the specified target
    SignalEvent(oTarget, EventSpellCastAt(OBJECT_SELF, SPELL_CONTINUAL_FLAME, FALSE));

    // Apply the VFX impact and effects
    ApplyEffectToObject(DURATION_TYPE_PERMANENT, eLink, oTarget);
  }
}
